@@ -75,7 +75,6 @@ static void TraverseShape(GF_Node *node, void *rs, Bool is_destroy)
 
 	/*check traverse mode, and take care of switch-off flag*/
 	if (tr_state->traversing_mode==TRAVERSE_GET_BOUNDS) {
-		GF_Node *m;
 		tr_state->appear = (GF_Node *) shape->appearance;
 
 		/*this is done regardless of switch flag*/
@@ -83,7 +82,7 @@ static void TraverseShape(GF_Node *node, void *rs, Bool is_destroy)
 
 		if (tr_state->appear) {
 			/*apply line width*/
-			m = ((M_Appearance *)tr_state->appear)->material;
+			GF_Node *m = ((M_Appearance *)tr_state->appear)->material;
 			if (m && (gf_node_get_tag(m)==TAG_MPEG4_Material2D) ) {
 				DrawAspect2D asp;
 				Fixed width = 0;
@@ -181,7 +180,7 @@ static void TraverseCircle(GF_Node *node, void *rs, Bool is_destroy)
 		if (!stack->mesh) {
 			Fixed a = ((M_Circle *) node)->radius * 2;
 			stack->mesh = new_mesh();
-			mesh_new_ellipse(stack->mesh, a, a, tr_state->visual->compositor->high_speed);
+			mesh_new_ellipse(stack->mesh, a, a, tr_state->visual->compositor->fast);
 		}
 		visual_3d_draw_2d(stack, tr_state);
 		return;
@@ -236,7 +235,7 @@ static void TraverseEllipse(GF_Node *node, void *rs, Bool is_destroy)
 	case TRAVERSE_DRAW_3D:
 		if (!stack->mesh) {
 			stack->mesh = new_mesh();
-			mesh_new_ellipse(stack->mesh, ((M_Ellipse *) node)->radius.x * 2, ((M_Ellipse *) node)->radius.y * 2, tr_state->visual->compositor->high_speed);
+			mesh_new_ellipse(stack->mesh, ((M_Ellipse *) node)->radius.x * 2, ((M_Ellipse *) node)->radius.y * 2, tr_state->visual->compositor->fast);
 		}
 		visual_3d_draw_2d(stack, tr_state);
 		return;
@@ -318,6 +317,85 @@ static void rectangle_check_changes(GF_Node *node, Drawable *stack, GF_TraverseS
 		drawable_mark_modified(stack, tr_state);
 	}
 }
+
+Bool rectangle_check_adaptation(GF_Node *node, Drawable *stack, GF_TraverseState *tr_state)
+{
+	GF_TextureHandler *txh;
+	GF_MediaObjectVRInfo vrinfo;
+	s32 gaze_x, gaze_y;
+	Bool is_visible = GF_FALSE;
+	if (! tr_state->visual->compositor->gazer_enabled)
+		return GF_TRUE;
+
+	if (!tr_state->appear || ! ((M_Appearance *)tr_state->appear)->texture)
+		return GF_TRUE;
+	
+	txh = gf_sc_texture_get_handler( ((M_Appearance *) tr_state->appear)->texture );
+	if (!txh->stream) return GF_TRUE;
+
+	if (! gf_mo_get_srd_info(txh->stream, &vrinfo))
+		return GF_TRUE;
+
+	if (!vrinfo.srd_w && !vrinfo.srd_h && vrinfo.is_tiled_srd) {
+		if (txh->stream->srd_full_w && txh->stream->srd_full_h) {
+			gaze_x = tr_state->visual->compositor->gaze_x;
+			gaze_x *= txh->stream->srd_full_w;
+			gaze_x /= tr_state->visual->width;
+
+			gaze_y = tr_state->visual->compositor->gaze_y;
+			gaze_y *= txh->stream->srd_full_h;
+			gaze_y /= tr_state->visual->height;
+
+			gf_mo_hint_gaze(txh->stream, gaze_x, gaze_y);
+		}
+
+		return GF_TRUE;
+	}
+
+	gaze_x = tr_state->visual->compositor->gaze_x;
+	gaze_x *= vrinfo.srd_max_x;
+	gaze_x /= tr_state->visual->width;
+
+	gaze_y = tr_state->visual->compositor->gaze_y;
+	gaze_y *= vrinfo.srd_max_y;
+	gaze_y /= tr_state->visual->height;
+
+	//simple test condition: only keep the first row
+	if ((gaze_x>=vrinfo.srd_x) && (gaze_x<=vrinfo.srd_x+vrinfo.srd_w) && (gaze_y>=vrinfo.srd_y) && (gaze_y<=vrinfo.srd_y+vrinfo.srd_h)) {
+
+		GF_LOG(GF_LOG_INFO, GF_LOG_COMPOSE, ("[Compositor] Texture %d Partial plane is under gaze coord %d %d\n", txh->stream->OD_ID, tr_state->visual->compositor->gaze_x, tr_state->visual->compositor->gaze_y));
+		is_visible = GF_TRUE;
+	}
+
+	if (vrinfo.has_full_coverage) {
+		if (is_visible) {
+			if (!txh->is_open) {
+				GF_LOG(GF_LOG_INFO, GF_LOG_COMPOSE, ("[Compositor] Texture %d stoped on visible partial plane - starting it\n", txh->stream->OD_ID));
+				assert(txh->stream && txh->stream->odm);
+				txh->stream->odm->disable_buffer_at_next_play = GF_TRUE;
+
+				gf_sc_texture_play(txh, NULL);
+			}
+			if (! txh->data)  return GF_FALSE;
+			return GF_TRUE;
+		} else {
+			if (txh->is_open) {
+				GF_LOG(GF_LOG_INFO, GF_LOG_COMPOSE, ("[Compositor] Texure %d playing on hidden partial plane - stoping it\n", txh->stream->OD_ID));
+				gf_sc_texture_stop_no_unregister(txh);
+			}
+			return GF_FALSE;
+		}
+	} else {
+		if (is_visible) {
+			gf_mo_hint_quality_degradation(txh->stream, 0);
+			if (! txh->data)  return GF_FALSE;
+		} else {
+			gf_mo_hint_quality_degradation(txh->stream, 100);
+		}
+	}
+	return GF_TRUE;
+}
+
 static void TraverseRectangle(GF_Node *node, void *rs, Bool is_destroy)
 {
 	DrawableContext *ctx;
@@ -331,6 +409,9 @@ static void TraverseRectangle(GF_Node *node, void *rs, Bool is_destroy)
 
 	rectangle_check_changes(node, stack, tr_state);
 
+	if (! rectangle_check_adaptation(node, stack, tr_state))
+		return;
+
 	switch (tr_state->traversing_mode) {
 	case TRAVERSE_DRAW_2D:
 		compositor_2d_draw_rectangle(tr_state);
@@ -412,7 +493,7 @@ void curve2d_check_changes(GF_Node *node, Drawable *stack, GF_TraverseState *tr_
 	if (!coord) return;
 
 	stack->path->fineness = c2D->fineness;
-	if (tr_state->visual->compositor->high_speed)  stack->path->fineness /= 2;
+	if (tr_state->visual->compositor->fast)  stack->path->fineness /= 2;
 
 
 	pts = coord->point.vals;
@@ -709,6 +790,9 @@ static void TraverseBitWrapper(GF_Node *node, void *rs, Bool is_destroy)
 		gf_node_set_private(node, NULL);
 		return;
 	}
+#ifdef GPAC_ENABLE_COVERAGE
+	if (!rs) return;
+#endif
 
 	tr_state = (GF_TraverseState *)rs;
 	// Traverse the node here
@@ -720,8 +804,15 @@ void compositor_init_bitwrapper(GF_Compositor *compositor, GF_Node *node)
 {
 	M_BitWrapper *bit;
 	bit = (M_BitWrapper *)node;
+	if (!bit->node) return;
 	gf_node_set_private(node, gf_node_get_private(bit->node));
 	gf_node_set_callback_function(node, TraverseBitWrapper);
+
+#ifdef GPAC_ENABLE_COVERAGE
+	if (gf_sys_is_cov_mode()) {
+		TraverseBitWrapper(node, NULL, GF_FALSE);
+	}
+#endif
 }
 
 
