@@ -26,7 +26,6 @@
 #include "nodes_stacks.h"
 #include "visual_manager.h"
 #include <gpac/options.h>
-#include <gpac/terminal.h>
 
 
 #ifndef GPAC_DISABLE_3D
@@ -274,6 +273,11 @@ Bool gf_sc_fit_world_to_screen(GF_Compositor *compositor)
 	camera_set_vectors(cam, pos, cam->vp_orientation, cam->fieldOfView);
 	cam->position = diff;
 	camera_move_to(cam, pos, cam->target, cam->up);
+	if (!compositor->player) {
+		camera_stop_anim(cam);
+		camera_set_vectors(cam, cam->end_pos, cam->end_ori, cam->end_fov);
+	}
+
 	cam->examine_center = tr_state.bbox.center;
 	cam->flags |= CF_STORE_VP;
 	if (cam->z_far < dist) cam->z_far = 10*dist;
@@ -282,7 +286,61 @@ Bool gf_sc_fit_world_to_screen(GF_Compositor *compositor)
 	return 1;
 }
 
-//#define SCALE_NAV
+static void handle_mouse_move_3d(GF_Compositor *compositor, GF_Camera *cam, u32 keys, Fixed dx, Fixed dy)
+{
+	Fixed trans_scale = cam->width/20;
+	//if default VP is quite far from center use larger dz/dy moves
+	if (cam->vp_dist>100) trans_scale *= 10;
+
+	switch (cam->navigate_mode) {
+	/*FIXME- we'll likely need a "step" value for walk at some point*/
+	case GF_NAVIGATE_WALK:
+	case GF_NAVIGATE_FLY:
+		view_pan_x(compositor, cam, -dx);
+		if (keys & GF_KEY_MOD_CTRL) view_pan_y(compositor, cam, dy);
+		else view_translate_z(compositor, cam, gf_mulfix(dy, trans_scale));
+		break;
+	case GF_NAVIGATE_VR:
+		view_pan_x(compositor, cam, -dx);
+		if (keys & GF_KEY_MOD_CTRL) view_zoom(compositor, cam, dy);
+		else view_pan_y(compositor, cam, dy);
+		break;
+	case GF_NAVIGATE_PAN:
+		view_pan_x(compositor, cam, -dx);
+		if (keys & GF_KEY_MOD_CTRL) view_translate_z(compositor, cam, gf_mulfix(dy, trans_scale));
+		else view_pan_y(compositor, cam, dy);
+		break;
+	case GF_NAVIGATE_SLIDE:
+		view_translate_x(compositor, cam, gf_mulfix(dx, trans_scale));
+		if (keys & GF_KEY_MOD_CTRL) view_translate_z(compositor, cam, gf_mulfix(dy, trans_scale));
+		else view_translate_y(compositor, cam, gf_mulfix(dy, trans_scale));
+		break;
+	case GF_NAVIGATE_EXAMINE:
+		if (keys & GF_KEY_MOD_CTRL) {
+			view_translate_z(compositor, cam, gf_mulfix(dy, trans_scale));
+			view_roll(compositor, cam, gf_mulfix(dx, trans_scale));
+		} else {
+			if (ABS(dx) > ABS(dy)) {
+				view_exam_x(compositor, cam, -gf_mulfix(GF_PI, dx));
+			} else {
+				view_exam_y(compositor, cam, gf_mulfix(GF_PI, dy));
+			}
+		}
+		break;
+	case GF_NAVIGATE_ORBIT:
+		if (keys & GF_KEY_MOD_CTRL) {
+			view_translate_z(compositor, cam, gf_mulfix(dy, trans_scale));
+		} else {
+			view_orbit_x(compositor, cam, -gf_mulfix(GF_PI, dx));
+			view_orbit_y(compositor, cam, gf_mulfix(GF_PI, dy));
+		}
+		break;
+	case GF_NAVIGATE_GAME:
+		view_pan_x(compositor, cam, -dx);
+		view_pan_y(compositor, cam, dy);
+		break;
+	}
+}
 
 static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event *ev)
 {
@@ -290,9 +348,6 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 	Fixed dx, dy, key_trans, key_pan, key_exam;
 	s32 key_inv;
 	u32 keys;
-#ifdef SCALE_NAV
-	Bool is_pixel_metrics;
-#endif
 	GF_Camera *cam;
 	Fixed zoom = compositor->zoom;
 
@@ -300,9 +355,6 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 #ifndef GPAC_DISABLE_VRML
 	if (compositor->active_layer) {
 		cam = compositor_layer3d_get_camera(compositor->active_layer);
-#ifdef SCALE_NAV
-		is_pixel_metrics = gf_sg_use_pixel_metrics(gf_node_get_graph(compositor->active_layer));
-#endif
 	}
 #endif
 
@@ -310,9 +362,6 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 		cam = &compositor->visual->camera;
 		assert(compositor);
 		assert(compositor->scene);
-#ifdef SCALE_NAV
-		is_pixel_metrics = compositor->traverse_state->pixel_metrics;
-#endif
 	}
 	if (!cam || (cam->navigate_mode==GF_NAVIGATE_NONE)) return 0;
 
@@ -328,21 +377,16 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 	dx = (x - compositor->grab_x);
 	dy = (compositor->grab_y - y);
 
-#ifdef SCALE_NAV
-	trans_scale = is_pixel_metrics ? cam->width/2 : INT2FIX(10);
-	key_trans = is_pixel_metrics ? INT2FIX(10) : cam->avatar_size.x;
-#else
 	trans_scale = cam->width/20;
 	key_trans = cam->avatar_size.x/2;
 	//if default VP is quite far from center use larger dz/dy moves
 	if (cam->vp_dist>100) trans_scale *= 10;
-#endif
 
 	if (cam->world_bbox.is_set && (key_trans*5 > cam->world_bbox.radius)) {
 		key_trans = cam->world_bbox.radius / 100;
 	}
 
-	key_pan = FIX_ONE/25;
+	key_pan = FIX_ONE/20;
 	key_exam = FIX_ONE/20;
 	key_inv = 1;
 
@@ -354,6 +398,14 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 		key_trans*=4;
 	}
 
+	if (! compositor->orientation_sensors_active) {
+		Fixed yaw, pitch, roll;
+		gf_mx_get_yaw_pitch_roll(&compositor->visual->camera.modelview, &yaw, &pitch, &roll);
+		compositor->audio_renderer->yaw = yaw;
+		compositor->audio_renderer->pitch = pitch;
+		compositor->audio_renderer->roll = roll;
+	}
+
 	switch (ev->type) {
 	case GF_EVENT_MOUSEDOWN:
 		/*left*/
@@ -383,9 +435,9 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 		}
 		break;
 
-	/* note: shortcuts are mostly the same as blaxxun contact, I don't feel like remembering 2 sets...*/
 	case GF_EVENT_MOUSEMOVE:
-		if (gf_term_get_option(compositor->term, GF_OPT_ORIENTATION_SENSORS_ACTIVE)) return 0;
+		if (compositor->orientation_sensors_active) return 0;
+
 		if (!compositor->navigation_state) {
 			if (cam->navigate_mode==GF_NAVIGATE_GAME) {
 				/*init mode*/
@@ -393,58 +445,19 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 				compositor->grab_y = y;
 				compositor->navigation_state = 1;
 			}
+			compositor->auto_rotate=0;
 			return 0;
 		}
 		compositor->navigation_state++;
 
-		switch (cam->navigate_mode) {
-		/*FIXME- we'll likely need a "step" value for walk at some point*/
-		case GF_NAVIGATE_WALK:
-		case GF_NAVIGATE_FLY:
-			view_pan_x(compositor, cam, -dx);
-			if (keys & GF_KEY_MOD_CTRL) view_pan_y(compositor, cam, dy);
-			else view_translate_z(compositor, cam, gf_mulfix(dy, trans_scale));
-			break;
-		case GF_NAVIGATE_VR:
-			view_pan_x(compositor, cam, -dx);
-			if (keys & GF_KEY_MOD_CTRL) view_zoom(compositor, cam, dy);
-			else view_pan_y(compositor, cam, dy);
-			break;
-		case GF_NAVIGATE_PAN:
-			view_pan_x(compositor, cam, -dx);
-			if (keys & GF_KEY_MOD_CTRL) view_translate_z(compositor, cam, gf_mulfix(dy, trans_scale));
-			else view_pan_y(compositor, cam, dy);
-			break;
-		case GF_NAVIGATE_SLIDE:
-			view_translate_x(compositor, cam, gf_mulfix(dx, trans_scale));
-			if (keys & GF_KEY_MOD_CTRL) view_translate_z(compositor, cam, gf_mulfix(dy, trans_scale));
-			else view_translate_y(compositor, cam, gf_mulfix(dy, trans_scale));
-			break;
-		case GF_NAVIGATE_EXAMINE:
-			if (keys & GF_KEY_MOD_CTRL) {
-				view_translate_z(compositor, cam, gf_mulfix(dy, trans_scale));
-				view_roll(compositor, cam, gf_mulfix(dx, trans_scale));
-			} else {
-				if (ABS(dx) > ABS(dy)) {
-					view_exam_x(compositor, cam, -gf_mulfix(GF_PI, dx));
-				} else {
-					view_exam_y(compositor, cam, gf_mulfix(GF_PI, dy));
-				}
-			}
-			break;
-		case GF_NAVIGATE_ORBIT:
-			if (keys & GF_KEY_MOD_CTRL) {
-				view_translate_z(compositor, cam, gf_mulfix(dy, trans_scale));
-			} else {
-				view_orbit_x(compositor, cam, -gf_mulfix(GF_PI, dx));
-				view_orbit_y(compositor, cam, gf_mulfix(GF_PI, dy));
-			}
-			break;
-		case GF_NAVIGATE_GAME:
-			view_pan_x(compositor, cam, -dx);
-			view_pan_y(compositor, cam, dy);
-			break;
-		}
+		if (x <= -0.49) compositor->auto_rotate = 1;
+		else if (x >= 0.49) compositor->auto_rotate = 2;
+		else if (y <= -0.49) compositor->auto_rotate = 3;
+		else if (y >= 0.49) compositor->auto_rotate = 4;
+		else compositor->auto_rotate = 0;
+
+		handle_mouse_move_3d(compositor, cam, keys, dx, dy);
+
 		compositor->grab_x = x;
 		compositor->grab_y = y;
 		return 1;
@@ -471,7 +484,36 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 		}
 		return 1;
 
+	case GF_EVENT_MULTITOUCH:
+		compositor->auto_rotate=0;
+		compositor->navigation_state = 0;
+		if (ev->mtouch.num_fingers==2) {
+			if( ABS(ev->mtouch.pinch) * 100 > 2 ) {
+				if (cam->is_3D) {
+					view_translate_z(compositor, cam, gf_mulfix(cam->width, compositor->visual->width* ev->mtouch.pinch));
+				} else {
+					nav_set_zoom_trans_2d(compositor->visual, zoom + gf_mulfix(trans_scale, ev->mtouch.pinch), 0, 0);
+				}
+				return 1;
+			}
+			if( ABS(ev->mtouch.rotation) > GF_PI/40 ) {
+				view_roll(compositor, cam, gf_mulfix(ev->mtouch.rotation, trans_scale));
+				return 1;
+			}
+		} else if (ev->mtouch.num_fingers==3) {
+			compositor->visual->camera.start_zoom = compositor->zoom;
+			compositor->zoom = FIX_ONE;
+			compositor->interoccular_offset = 0;
+			compositor->focdist = 0;
+			compositor->interoccular_offset = 0;
+			compositor->focdist = 0;
+			compositor_3d_reset_camera(compositor);
+			return 1;
+		}
+		return 0;
+
 	case GF_EVENT_MOUSEUP:
+		compositor->auto_rotate=0;
 		if (ev->mouse.button==GF_MOUSE_LEFT) compositor->navigation_state = 0;
 		break;
 
@@ -495,9 +537,9 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 				compositor->visual->camera.start_zoom = compositor->zoom;
 				compositor->zoom = FIX_ONE;
 				compositor->interoccular_offset = 0;
-				compositor->focus_distance = 0;
+				compositor->focdist = 0;
 				compositor->interoccular_offset = 0;
-				compositor->focus_distance = 0;
+				compositor->focdist = 0;
 				compositor_3d_reset_camera(compositor);
 			}
 			break;
@@ -514,9 +556,9 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 			if (keys & GF_KEY_MOD_ALT) {
 				if ( (keys & GF_KEY_MOD_SHIFT) && (compositor->visual->nb_views > 1) ) {
 					/*+ or - 10 cm*/
-					compositor->focus_distance += INT2FIX(key_inv);
+					compositor->focdist += INT2FIX(key_inv);
 					cam->flags |= CAM_IS_DIRTY;
-					fprintf(stderr, "AutoStereo view distance %f - focus %f\n", FIX2FLT(compositor->video_out->view_distance)/100, FIX2FLT(compositor->focus_distance)/100);
+					fprintf(stderr, "AutoStereo view distance %f - focus %f\n", FIX2FLT(compositor->video_out->dispdist)/100, FIX2FLT(compositor->focdist)/100);
 					gf_sc_invalidate(compositor, NULL);
 					return 1;
 				}
@@ -556,7 +598,7 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 			if (keys & GF_KEY_MOD_ALT) {
 				if ( (keys & GF_KEY_MOD_SHIFT) && (compositor->visual->nb_views > 1) ) {
 					compositor->interoccular_offset += FLT2FIX(0.5) * key_inv;
-					fprintf(stderr, "AutoStereo interoccular distance %f\n", FIX2FLT(compositor->interoccular_distance + compositor->interoccular_offset));
+					fprintf(stderr, "AutoStereo interoccular distance %f\n", FIX2FLT(compositor->iod + compositor->interoccular_offset));
 					cam->flags |= CAM_IS_DIRTY;
 					gf_sc_invalidate(compositor, NULL);
 					return 1;
@@ -607,35 +649,55 @@ static Bool compositor_handle_navigation_3d(GF_Compositor *compositor, GF_Event
 				return 1;
 			}
 			break;
+		case GF_KEY_D:
+			if (keys & GF_KEY_MOD_CTRL) {
+				compositor->tvtd = !compositor->tvtd;
+				gf_sc_invalidate(compositor, NULL);
+				return 1;
+			}
+			break;
+		case GF_KEY_G:
+			if (keys & GF_KEY_MOD_CTRL) {
+				compositor->vrhud_mode++;
+				if (compositor->vrhud_mode==5) compositor->vrhud_mode=0;
+				gf_sc_invalidate(compositor, NULL);
+				return 1;
+			}
+			break;
+		case GF_KEY_A:
+			if (keys & GF_KEY_MOD_CTRL) {
+				compositor->tvtf = !compositor->tvtf;
+				gf_sc_invalidate(compositor, NULL);
+				return 1;
+			}
+			break;
 		}
 		break;
 	case GF_EVENT_SENSOR_ORIENTATION:
 	{
-		Fixed x, y, z, w, yaw, /*pitch, */roll;
+		Fixed z, w, yaw, /*pitch, */roll;
 		GF_Vec target;
 		GF_Matrix mx;
 
-#ifndef GPAC_ANDROID
-		/*
-		 * In iOS we get x, y, z in quaternions (first measurement is the frame of reference)
-		 */
-		x = ev->sensor.x;
-		y = ev->sensor.y;
-		z = ev->sensor.z;
-		w = ev->sensor.w;
+		/* In iOS we get x, y, z in quaternions (first measurement is the frame of reference) */
+		if (ev->sensor.w) {
+			x = ev->sensor.x;
+			y = ev->sensor.y;
+			z = ev->sensor.z;
+			w = ev->sensor.w;
 		
-		yaw = gf_atan2(2*gf_mulfix(z,w) - 2*gf_mulfix(y,x) , 1 - 2*gf_mulfix(z,z) - 2*gf_mulfix(x,x));
-		//pitch = asin(2*y*z + 2*x*w);
-		roll = gf_atan2(2*gf_mulfix(y,w) - 2*gf_mulfix(z,x) , 1 - 2*gf_mulfix(y,y) - 2*gf_mulfix(x,x));
-#else
-		/*
-		 * In Android we get yaw, pitch, roll values (in rad)
-		 * The frame of reference is absolute
-		 */
-		yaw = ev->sensor.x;
-		//pitch = ev->sensor.y;
-		roll = ev->sensor.z;
-#endif
+			yaw = gf_atan2(2*gf_mulfix(z,w) - 2*gf_mulfix(y,x) , 1 - 2*gf_mulfix(z,z) - 2*gf_mulfix(x,x));
+			//pitch = asin(2*y*z + 2*x*w);
+			roll = gf_atan2(2*gf_mulfix(y,w) - 2*gf_mulfix(z,x) , 1 - 2*gf_mulfix(y,y) - 2*gf_mulfix(x,x));
+		} else {
+			/*
+			 * In Android we get yaw, pitch, roll values (in rad)
+			 * The frame of reference is absolute
+			 */
+			yaw = ev->sensor.x;
+			//pitch = ev->sensor.y;
+			roll = ev->sensor.z;
+		}
 		target.x = 0;
 		target.y = -FIX_ONE;
 		target.z = 0;
@@ -729,6 +791,26 @@ static Bool compositor_handle_navigation_2d(GF_VisualManager *visual, GF_Event *
 			return 0;
 		}
 		break;
+	case GF_EVENT_MULTITOUCH:
+		if (ev->mtouch.num_fingers==2) {
+			if( ABS(ev->mtouch.pinch) * 100 > 2 ) {
+				new_zoom = zoom + ev->mtouch.pinch * MIN(visual->width, visual->height)/100;
+				nav_set_zoom_trans_2d(visual, new_zoom, 0, 0);
+				return 1;
+			}
+			if( ABS(ev->mtouch.rotation) > GF_PI/40 ) {
+				visual->compositor->rotation -= ev->mtouch.rotation;
+				nav_set_zoom_trans_2d(visual, zoom, 0, 0);
+				return 1;
+			}
+		} else if (ev->mtouch.num_fingers==3) {
+			visual->compositor->trans_x = visual->compositor->trans_y = 0;
+			visual->compositor->rotation = 0;
+			visual->compositor->zoom = FIX_ONE;
+			nav_set_zoom_trans_2d(visual, FIX_ONE, 0, 0);
+			return 1;
+		}
+		return 0;
 
 	case GF_EVENT_MOUSEWHEEL:
 		switch (navigation_mode) {
@@ -807,7 +889,7 @@ static Bool compositor_handle_navigation_2d(GF_VisualManager *visual, GF_Event *
 		case GF_KEY_UP:
 			if (navigation_mode == GF_NAVIGATE_SLIDE) {
 				if (keys & GF_KEY_MOD_CTRL) {
-					Fixed new_zoom = zoom;
+					new_zoom = zoom;
 					if (new_zoom > FIX_ONE) new_zoom += key_inv*FIX_ONE/10;
 					else new_zoom += key_inv*FIX_ONE/20;
 					nav_set_zoom_trans_2d(visual, new_zoom, 0, 0);
@@ -826,6 +908,40 @@ static Bool compositor_handle_navigation_2d(GF_VisualManager *visual, GF_Event *
 	return 0;
 }
 
+void compositor_handle_auto_navigation(GF_Compositor *compositor)
+{
+#ifndef GPAC_DISABLE_3D
+	GF_Camera *cam = NULL;
+	Fixed dx, dy;
+#ifndef GPAC_DISABLE_VRML
+	if (compositor->active_layer) {
+		cam = compositor_layer3d_get_camera(compositor->active_layer);
+	}
+	if (!cam)
+		cam = &compositor->visual->camera;
+#endif
+
+	dx = dy = 0;
+	switch (compositor->auto_rotate) {
+	case 1:
+		dx = -FIX_ONE/100;
+		break;
+	case 2:
+		dx = FIX_ONE/100;
+		break;
+	case 3:
+		dy = FIX_ONE/100;
+		break;
+	case 4:
+		dy = -FIX_ONE/100;
+		break;
+	default:
+		return;
+	}
+	handle_mouse_move_3d(compositor, cam, 0, dx, dy);
+#endif
+}
+
 Bool compositor_handle_navigation(GF_Compositor *compositor, GF_Event *ev)
 {
 	if (!compositor->scene) return 0;
