@@ -102,7 +102,7 @@ void group_cache_setup(GroupCache *cache, GF_Rect *path_bounds, GF_IRect *pix_bo
 		gf_free(cache->txh.data);
 #ifdef CACHE_DEBUG_ALPHA
 	cache->txh.stride = pix_bounds->width * 3;
-	cache->txh.pixelformat = GF_PIXEL_RGB_24;
+	cache->txh.pixelformat = GF_PIXEL_RGB;
 	cache->txh.transparent = 0;
 #endif
 
@@ -139,13 +139,11 @@ Bool group_cache_traverse(GF_Node *node, GroupCache *cache, GF_TraverseState *tr
 
 	/*we need to redraw the group in an offscreen visual*/
 	if (force_recompute) {
-		GF_Matrix2D backup;
 		GF_IRect rc1, rc2;
 		u32 prev_flags;
 		Bool prev_hybgl, visual_attached, for_3d=GF_FALSE;
 		GF_Rect cache_bounds;
-		GF_SURFACE offscreen_surface, old_surf;
-		GF_Raster2D *r2d = tr_state->visual->compositor->rasterizer;
+		GF_EVGSurface *offscreen_surface, *old_surf;
 		DrawableContext *child_ctx;
 		Fixed temp_x, temp_y, scale_x, scale_y;
 #ifndef GPAC_DISABLE_3D
@@ -214,7 +212,7 @@ Bool group_cache_traverse(GF_Node *node, GroupCache *cache, GF_TraverseState *tr
 			setup top clipers
 		*/
 		old_surf = tr_state->visual->raster_surface;
-		offscreen_surface = r2d->surface_new(r2d, tr_state->visual->center_coords);	/*a new temp raster visual*/
+		offscreen_surface = gf_evg_surface_new(tr_state->visual->center_coords);	/*a new temp raster visual*/
 		tr_state->visual->raster_surface = offscreen_surface;
 #ifndef GPAC_DISABLE_3D
 		if (type_3d) {
@@ -230,8 +228,8 @@ Bool group_cache_traverse(GF_Node *node, GroupCache *cache, GF_TraverseState *tr
 
 		/*use current surface coordinate scaling to compute the cache*/
 #ifdef GF_SR_USE_VIDEO_CACHE
-		scale_x = tr_state->visual->compositor->cache_scale * scale_x / 100;
-		scale_y = tr_state->visual->compositor->cache_scale * scale_y / 100;
+		scale_x = tr_state->visual->compositor->vcscale * scale_x / 100;
+		scale_y = tr_state->visual->compositor->vcscale * scale_y / 100;
 #endif
 
 		if (scale_x<0) scale_x = -scale_x;
@@ -256,7 +254,7 @@ Bool group_cache_traverse(GF_Node *node, GroupCache *cache, GF_TraverseState *tr
 
 
 		/*attach the buffer to visual*/
-		r2d->surface_attach_to_buffer(offscreen_surface, cache->txh.data,
+		gf_evg_surface_attach_to_buffer(offscreen_surface, cache->txh.data,
 		                              cache->txh.width,
 		                              cache->txh.height,
 		                              0,
@@ -327,7 +325,7 @@ Bool group_cache_traverse(GF_Node *node, GroupCache *cache, GF_TraverseState *tr
 		tr_state->visual->compositor->hybrid_opengl = prev_hybgl;
 		tr_state->visual->is_attached = visual_attached;
 
-		r2d->surface_delete(offscreen_surface);
+		gf_evg_surface_delete(offscreen_surface);
 		tr_state->visual->raster_surface = old_surf;
 		tr_state->traversing_mode = TRAVERSE_SORT;
 
@@ -448,7 +446,7 @@ static void group_cache_insert_entry(GF_Node *node, GroupingNode2D *group, GF_Tr
 	                                    group->traverse_time));
 
 	GF_LOG(GF_LOG_DEBUG, GF_LOG_CACHE, ("[CACHE] Status (KB): Max: %d\tUsed: %d\tNb Groups: %d\n",
-	                                    tr_state->visual->compositor->video_cache_max_size,
+	                                    tr_state->visual->compositor->vcsize,
 	                                    tr_state->visual->compositor->video_cache_current_size,
 	                                    gf_list_count(tr_state->visual->compositor->cached_groups)
 	                                   ));
@@ -500,7 +498,7 @@ static Bool gf_cache_remove_entry(GF_Compositor *compositor, GF_Node *node, Grou
 	                                    FIX2FLT(group->traverse_time)));
 
 	GF_LOG(GF_LOG_DEBUG, GF_LOG_CACHE, ("[CACHE] Status (B): Max: %d\tUsed: %d\tNb Groups: %d\n",
-	                                    compositor->video_cache_max_size,
+	                                    compositor->vcsize,
 	                                    compositor->video_cache_current_size,
 	                                    gf_list_count(compositor->cached_groups)
 	                                   ));
@@ -605,9 +603,9 @@ Bool group_2d_cache_traverse(GF_Node *node, GroupingNode2D *group, GF_TraverseSt
 		else if (group->cache) {
 			Fixed scale = MAX(tr_state->transform.m[0], tr_state->transform.m[4]);
 
-			if (100*scale >= group->cache->scale*(100 + tr_state->visual->compositor->cache_tolerance))
+			if (100*scale >= group->cache->scale*(100 + tr_state->visual->compositor->vctol))
 				zoom_changed = 1;
-			else if ((100+tr_state->visual->compositor->cache_tolerance)*scale <= 100*group->cache->scale)
+			else if ((100+tr_state->visual->compositor->vctol)*scale <= 100*group->cache->scale)
 				zoom_changed = 1;
 			else
 				zoom_changed = 0;
@@ -648,7 +646,7 @@ Bool group_cache_compute_stats(GF_Node *node, GroupingNode2D *group, GF_Traverse
 	DrawableContext *ctx;
 	u32 nb_segments, nb_objects;
 	u32 alpha_pixels, opaque_pixels, area_world;
-	u32 video_cache_max_size, cache_size, prev_cache_size;
+	u32 vcsize, cache_size, prev_cache_size;
 	u32 i;
 	GF_RectArray ra;
 
@@ -659,7 +657,7 @@ Bool group_cache_compute_stats(GF_Node *node, GroupingNode2D *group, GF_Traverse
 	prev_cache_size = group->cached_size;
 	/*reset bounds*/
 	group_bounds.width = group_bounds.height = 0;
-	video_cache_max_size = tr_state->visual->compositor->video_cache_max_size;
+	vcsize = tr_state->visual->compositor->vcsize;
 
 	/*never cache root node - this should be refined*/
 	if (gf_node_get_parent(node, 0) == NULL) goto group_reject;
@@ -726,14 +724,14 @@ Bool group_cache_compute_stats(GF_Node *node, GroupingNode2D *group, GF_Traverse
 	if (10*opaque_pixels < 7*area_world) goto group_reject;
 
 	/*the memory size allocated for the cache - cache is drawn in final coordinate system !!*/
-	group_bounds.width = tr_state->visual->compositor->cache_scale * group_bounds.width / 100;
-	group_bounds.height = tr_state->visual->compositor->cache_scale * group_bounds.height / 100;
+	group_bounds.width = tr_state->visual->compositor->vcscale * group_bounds.width / 100;
+	group_bounds.height = tr_state->visual->compositor->vcscale * group_bounds.height / 100;
 	cache_size = FIX2INT(group_bounds.width) * FIX2INT(group_bounds.height) * 4 /* pixelFormat is ARGB*/;
 
 	/*TEST 5: cache is less than 10x10 pixels: discard*/
 	if (cache_size < 400) goto group_reject;
 	/*TEST 6: cache is larger than our allowed memory: discard*/
-	if (cache_size>=video_cache_max_size) {
+	if (cache_size>=vcsize) {
 		tr_state->cache_too_small = 1;
 		goto group_reject;
 	}
@@ -795,7 +793,7 @@ group_reject:
 	                                   ));
 
 	GF_LOG(GF_LOG_DEBUG, GF_LOG_CACHE, ("[CACHE] Status (B): Max: %d\tUsed: %d\tNb Groups: %d\n",
-	                                    tr_state->visual->compositor->video_cache_max_size,
+	                                    tr_state->visual->compositor->vcsize,
 	                                    tr_state->visual->compositor->video_cache_current_size,
 	                                    gf_list_count(tr_state->visual->compositor->cached_groups)
 	                                   ));
@@ -807,12 +805,11 @@ group_reject:
 void group_2d_cache_evaluate(GF_Node *node, GroupingNode2D *group, GF_TraverseState *tr_state, DrawableContext *first_child, Bool skip_first_child, u32 last_cache_idx)
 {
 	u32 nb_cache_added, i;
-	Fixed priority;
 	GF_Compositor *compositor = tr_state->visual->compositor;
 
 	/*first frame is unusable for stats because lot of time is being spent building the path and allocating
 	the drawable contexts*/
-	if (!compositor->video_cache_max_size || !compositor->frame_number || group->changed || tr_state->in_group_cache) {
+	if (!compositor->vcsize || !compositor->frame_number || group->changed || tr_state->in_group_cache) {
 		group->traverse_time = 0;
 		return;
 	}
@@ -852,10 +849,10 @@ void group_2d_cache_evaluate(GF_Node *node, GroupingNode2D *group, GF_TraverseSt
 			cache->traverse_time = 0;
 			gf_list_rem(compositor->cached_groups_queue, last_cache_idx);
 		}
-		priority = INT2FIX (group->nb_objects*1024*1024*avg_time) / group->cached_size;
+		group->priority = INT2FIX (group->nb_objects*1024*1024*avg_time) / group->cached_size;
 
 		/*when the memory exceeds the constraint, remove the subgroups that have the lowest deltas*/
-		while (compositor->video_cache_current_size > compositor->video_cache_max_size)	{
+		while (compositor->video_cache_current_size > compositor->vcsize)	{
 			gf_cache_remove_entry(compositor, node, NULL);
 			GF_LOG(GF_LOG_DEBUG, GF_LOG_CACHE, ("[CACHE] Removing low priority cache - current total size %d\n", compositor->video_cache_current_size));
 		}
@@ -868,7 +865,7 @@ void compositor_set_cache_memory(GF_Compositor *compositor, u32 memory)
 	while (compositor->video_cache_current_size) {
 		gf_cache_remove_entry(compositor, NULL, NULL);
 	}
-	compositor->video_cache_max_size = memory;
+	compositor->vcsize = memory;
 	/*and force recompute*/
 	compositor->zoom_changed = 1;
 }
