@@ -31,7 +31,7 @@
 
 static void rtp_amr_flush(GP_RTPPacketizer *builder)
 {
-	char *hdr;
+	u8 *hdr;
 	u32 hdr_size;
 	if (!builder->bytesInPacket) return;
 	gf_bs_get_content(builder->pck_hdr, &hdr, &hdr_size);
@@ -46,7 +46,7 @@ static void rtp_amr_flush(GP_RTPPacketizer *builder)
 	builder->last_au_sn = 0;
 }
 
-GF_Err gp_rtp_builder_do_amr(GP_RTPPacketizer *builder, char *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
+GF_Err gp_rtp_builder_do_amr(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
 {
 	u32 offset, rtp_ts, block_size;
 
@@ -128,7 +128,7 @@ static GFINLINE u8 qes_get_rate_size(u32 idx, const unsigned int *rates, const u
 	return 0;
 }
 
-GF_Err gp_rtp_builder_do_qcelp(GP_RTPPacketizer *builder, char *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
+GF_Err gp_rtp_builder_do_qcelp(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
 {
 	u32 offset, rtp_ts;
 	u8 hdr;
@@ -190,12 +190,11 @@ GF_Err gp_rtp_builder_do_qcelp(GP_RTPPacketizer *builder, char *data, u32 data_s
 	return GF_OK;
 }
 
-
 static void rtp_evrc_smv_flush(GP_RTPPacketizer *builder)
 {
 	if (!builder->bytesInPacket) return;
 	if (builder->auh_size>1) {
-		char *hdr;
+		u8 *hdr;
 		u32 hdr_size;
 		/*padding*/
 		if (builder->last_au_sn % 2) gf_bs_write_int(builder->pck_hdr, 0, 4);
@@ -213,7 +212,7 @@ static void rtp_evrc_smv_flush(GP_RTPPacketizer *builder)
 	builder->last_au_sn = 0;
 }
 
-GF_Err gp_rtp_builder_do_smv(GP_RTPPacketizer *builder, char *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
+GF_Err gp_rtp_builder_do_smv(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
 {
 	u32 offset, rtp_ts;
 
@@ -250,7 +249,7 @@ GF_Err gp_rtp_builder_do_smv(GP_RTPPacketizer *builder, char *data, u32 data_siz
 				builder->pck_hdr = gf_bs_new(NULL, 0, GF_BITSTREAM_WRITE);
 				/*RRLLLNNN (all 0, no interleaving)*/
 				gf_bs_write_u8(builder->pck_hdr, 0);
-				/*MMM + count-1 : overriden when flushing*/
+				/*MMM + count-1 : overridden when flushing*/
 				gf_bs_write_u8(builder->pck_hdr, 0);
 				builder->bytesInPacket = 2;
 			}
@@ -282,10 +281,9 @@ GF_Err gp_rtp_builder_do_smv(GP_RTPPacketizer *builder, char *data, u32 data_siz
 	return GF_OK;
 }
 
-GF_Err gp_rtp_builder_do_h263(GP_RTPPacketizer *builder, char *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
+GF_Err gp_rtp_builder_do_h263(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
 {
-	GF_BitStream *bs;
-	char hdr[2];
+	u8 hdr[2];
 	Bool Pbit;
 	u32 offset, size, max_size;
 
@@ -302,6 +300,7 @@ GF_Err gp_rtp_builder_do_h263(GP_RTPPacketizer *builder, char *data, u32 data_si
 	max_size = builder->Path_MTU - 2;
 
 	while(data_size > 0) {
+		GF_BitStream *bs;
 		if(data_size > max_size) {
 			size = max_size;
 			builder->rtp_header.Marker = 0;
@@ -338,10 +337,46 @@ GF_Err gp_rtp_builder_do_h263(GP_RTPPacketizer *builder, char *data, u32 data_si
 	return GF_OK;
 }
 
-GF_Err gp_rtp_builder_do_tx3g(GP_RTPPacketizer *builder, char *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize, u32 duration, u8 descIndex)
+GF_Err gp_rtp_builder_do_mp2t(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
+{
+	u32 offset, size, max_size;
+
+	builder->rtp_header.TimeStamp = (u32) builder->sl_header.compositionTimeStamp;
+
+	if (!data) return GF_OK;
+
+	max_size = builder->Path_MTU;
+	offset = 0;
+	while (data_size > 0) {
+		if (data_size > max_size) {
+			size = max_size / 188;
+			size *= 188;
+		} else {
+			size = data_size;
+		}
+
+		data_size -= size;
+
+		/*create new RTP Packet */
+		builder->rtp_header.SequenceNumber += 1;
+		builder->OnNewPacket(builder->cbk_obj, &builder->rtp_header);
+
+		/*add payload*/
+		if (builder->OnDataReference)
+			builder->OnDataReference(builder->cbk_obj, size, offset);
+		else
+			builder->OnData(builder->cbk_obj, data + offset, size, GF_TRUE);
+
+		builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
+
+		offset += size;
+	}
+	return GF_OK;
+}
+GF_Err gp_rtp_builder_do_tx3g(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize, u32 duration, u8 descIndex)
 {
 	GF_BitStream *bs;
-	char *hdr;
+	u8 *hdr;
 	u32 samp_size, txt_size, pay_start, hdr_size, txt_done, cur_frag, nb_frag;
 	Bool is_utf_16 = GF_FALSE;
 
@@ -374,6 +409,8 @@ GF_Err gp_rtp_builder_do_tx3g(GP_RTPPacketizer *builder, char *data, u32 data_si
 		builder->OnPacketDone(builder->cbk_obj, &builder->rtp_header);
 		builder->bytesInPacket = 0;
 	}
+	//we only deal with static SIDX
+	descIndex += GF_RTP_TX3G_SIDX_OFFSET;
 
 	/*first TTU in packet*/
 	if (!builder->bytesInPacket) {
@@ -544,6 +581,7 @@ GF_Err gp_rtp_builder_do_tx3g(GP_RTPPacketizer *builder, char *data, u32 data_si
 }
 
 
+#if GPAC_ENABLE_3GPP_DIMS_RTP
 GF_Err gp_rtp_builder_do_dims(GP_RTPPacketizer *builder, char *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize, u32 duration)
 {
 	u32 frag_state;
@@ -667,6 +705,7 @@ GF_Err gp_rtp_builder_do_dims(GP_RTPPacketizer *builder, char *data, u32 data_si
 	gf_bs_del(bs);
 	return GF_OK;
 }
+#endif
 
 
 
@@ -685,7 +724,7 @@ static void gf_rtp_ac3_flush(GP_RTPPacketizer *builder)
 	builder->ac3_ft = 0;
 }
 
-GF_Err gp_rtp_builder_do_ac3(GP_RTPPacketizer *builder, char *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
+GF_Err gp_rtp_builder_do_ac3(GP_RTPPacketizer *builder, u8 *data, u32 data_size, u8 IsAUEnd, u32 FullAUSize)
 {
 	char hdr[2];
 	u32 offset, nb_pck;
