@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre
- *			Copyright (c) Telecom ParisTech 2000-2012
+ *			Copyright (c) Telecom ParisTech 2000-2020
  *					All rights reserved
  *
  *  This file is part of GPAC / MPEG-4 ObjectDescriptor sub-project
@@ -91,7 +91,6 @@ u32 gf_odf_get_tag_by_name(char *descName)
 	if (!stricmp(descName, "DecoderSpecificInfoString")) return GF_ODF_DSI_TAG;
 	if (!stricmp(descName, "SLConfigDescriptor")) return GF_ODF_SLC_TAG;
 	if (!stricmp(descName, "SegmentDescriptor")) return GF_ODF_SEGMENT_TAG;
-	if (!stricmp(descName, "MediaTimeDescriptor")) return GF_ODF_MEDIATIME_TAG;
 	if (!stricmp(descName, "MuxInfo")) return GF_ODF_MUXINFO_TAG;
 	if (!stricmp(descName, "StreamSource")) return GF_ODF_MUXINFO_TAG;
 	if (!stricmp(descName, "BIFSConfig") || !stricmp(descName, "BIFSv2Config")) return GF_ODF_BIFS_CFG_TAG;
@@ -103,8 +102,11 @@ u32 gf_odf_get_tag_by_name(char *descName)
 	if (!stricmp(descName, "ES_ID_Inc")) return GF_ODF_ESD_INC_TAG;
 	if (!stricmp(descName, "AuxiliaryVideoData")) return GF_ODF_AUX_VIDEO_DATA;
 	if (!stricmp(descName, "DefaultDescriptor")) return GF_ODF_DSI_TAG;
+	if (!stricmp(descName, "LanguageDescriptor")) return GF_ODF_LANG_TAG;
+	if (!stricmp(descName, "GPACLanguage")) return GF_ODF_GPAC_LANG;
 
 #ifndef GPAC_MINIMAL_ODF
+	if (!stricmp(descName, "MediaTimeDescriptor")) return GF_ODF_MEDIATIME_TAG;
 	if (!stricmp(descName, "ContentIdentification")) return GF_ODF_CI_TAG;
 	if (!stricmp(descName, "SuppContentIdentification")) return GF_ODF_SCI_TAG;
 	if (!stricmp(descName, "IPIPtr")) return GF_ODF_IPI_PTR_TAG;
@@ -119,7 +121,6 @@ u32 gf_odf_get_tag_by_name(char *descName)
 	if (!stricmp(descName, "ContentClassification")) return GF_ODF_CC_TAG;
 	if (!stricmp(descName, "KeyWordDescriptor")) return GF_ODF_KW_TAG;
 	if (!stricmp(descName, "RatingDescriptor")) return GF_ODF_RATING_TAG;
-	if (!stricmp(descName, "LanguageDescriptor")) return GF_ODF_LANG_TAG;
 	if (!stricmp(descName, "ShortTextualDescriptor")) return GF_ODF_SHORT_TEXT_TAG;
 	if (!stricmp(descName, "ExpandedTextualDescriptor")) return GF_ODF_TEXT_TAG;
 	if (!stricmp(descName, "ContentCreatorName")) return GF_ODF_CC_NAME_TAG;
@@ -133,14 +134,14 @@ u32 gf_odf_get_tag_by_name(char *descName)
 
 #include <gpac/internal/odf_parse_common.h>
 
-void OD_ParseBinData(char *val, char **out_data, u32 *out_data_size)
+void OD_ParseBinData(u8 *val, u8 **out_data, u32 *out_data_size)
 {
 	u32 i, c;
 	char s[3];
 	u32 len = (u32) strlen(val) / 3;
 	if (*out_data) gf_free(*out_data);
 	*out_data_size = len;
-	*out_data = (char*)gf_malloc(sizeof(char) * len);
+	*out_data = gf_malloc(sizeof(char) * len);
 	s[2] = 0;
 	for (i=0; i<len; i++) {
 		s[0] = val[3*i+1];
@@ -150,64 +151,13 @@ void OD_ParseBinData(char *val, char **out_data, u32 *out_data_size)
 	}
 }
 
-void OD_ParseFileData(char *fileName, char **out_data, u32 *out_data_size)
-{
-	FILE *f;
-	u32 size;
-	size_t readen;
-	if (*out_data) gf_free(*out_data);
-	*out_data = NULL;
-	*out_data_size = 0;
-	f = gf_fopen(fileName, "rb");
-	if (!f) {
-		GF_LOG(GF_LOG_WARNING, GF_LOG_PARSER, ("[ODF Parse] cannot open data file %s - skipping\n", fileName));
-		return;
-	}
-	gf_fseek(f, 0, SEEK_END);
-	assert(gf_ftell(f) < 1<<31);
-	size = (u32) gf_ftell(f);
-	gf_fseek(f, 0, SEEK_SET);
-	*out_data_size = size;
-	*out_data = (char*)gf_malloc(sizeof(char) * (size_t)size);
-	readen = fread(*out_data, sizeof(char), (size_t)size, f);
-	if (readen != size) {
-		GF_LOG(GF_LOG_WARNING, GF_LOG_PARSER, ("[ODF Parse] readen size=%d does not match size=%d in %s\n", readen, size, fileName));
-	}
-	gf_fclose(f);
-}
-
-void OD_ParseBin128(char *val, bin128 *data)
-{
-	if (!strnicmp(val, "0x", 2)) val+=2;
-
-	if (strlen(val)<16) {
-		GF_BitStream *bs;
-		u32 int_val = atoi(val);
-		bs = gf_bs_new((char*) (*data), 16, GF_BITSTREAM_WRITE);
-		gf_bs_write_int(bs, 0, 32);
-		gf_bs_write_int(bs, 0, 32);
-		gf_bs_write_int(bs, 0, 32);
-		gf_bs_write_int(bs, int_val, 32);
-		gf_bs_del(bs);
-	} else {
-		u32 i, b;
-		char szB[3];
-		szB[2] = 0;
-		for (i=0; i<16; i++) {
-			szB[0] = val[2*i];
-			szB[1] = val[2*i+1];
-			sscanf(szB, "%x", &b);
-			((char *)data)[i] = (u8) b;
-		}
-	}
-}
-
-
 GF_Err gf_odf_set_field(GF_Descriptor *desc, char *fieldName, char *val)
 {
-	Bool OD_ParseUIConfig(char *val, char **out_data, u32 *out_data_size);
+#ifndef GPAC_MINIMAL_ODF
+	Bool OD_ParseUIConfig(u8 *val, u8 **out_data, u32 *out_data_size);
+#endif
 	u32 ret = 0;
-
+	if (!fieldName || !val) return GF_BAD_PARAM;
 	if (!stricmp(val, "auto")) return GF_OK;
 	else if (!stricmp(val, "unspecified")) return GF_OK;
 
@@ -282,75 +232,75 @@ GF_Err gf_odf_set_field(GF_Descriptor *desc, char *fieldName, char *val)
 			/*XMT may use string*/
 			if (!ret) {
 				if (!stricmp(val, "MPEG4Systems1")) {
-					dcd->objectTypeIndication = GPAC_OTI_OD_V1;
+					dcd->objectTypeIndication = GF_CODECID_OD_V1;
 					ret = 1;
 				}
 				else if (!stricmp(val, "MPEG4Systems2")) {
-					dcd->objectTypeIndication = GPAC_OTI_SCENE_BIFS_V2;
+					dcd->objectTypeIndication = GF_CODECID_BIFS_V2;
 					ret = 1;
 				}
 				else if (!stricmp(val, "MPEG4Visual")) {
-					dcd->objectTypeIndication = GPAC_OTI_VIDEO_MPEG4_PART2;
+					dcd->objectTypeIndication = GF_CODECID_MPEG4_PART2;
 					ret = 1;
 				}
 				else if (!stricmp(val, "MPEG4Audio")) {
-					dcd->objectTypeIndication = GPAC_OTI_AUDIO_AAC_MPEG4;
+					dcd->objectTypeIndication = GF_CODECID_AAC_MPEG4;
 					ret = 1;
 				}
 				else if (!stricmp(val, "MPEG2VisualSimple")) {
-					dcd->objectTypeIndication = GPAC_OTI_VIDEO_MPEG2_SIMPLE;
+					dcd->objectTypeIndication = GF_CODECID_MPEG2_SIMPLE;
 					ret = 1;
 				}
 				else if (!stricmp(val, "MPEG2VisualMain")) {
-					dcd->objectTypeIndication = GPAC_OTI_VIDEO_MPEG2_MAIN;
+					dcd->objectTypeIndication = GF_CODECID_MPEG2_MAIN;
 					ret = 1;
 				}
 				else if (!stricmp(val, "MPEG2VisualSNR")) {
-					dcd->objectTypeIndication = GPAC_OTI_VIDEO_MPEG2_SNR;
+					dcd->objectTypeIndication = GF_CODECID_MPEG2_SNR;
 					ret = 1;
 				}
 				else if (!stricmp(val, "MPEG2VisualSpatial")) {
-					dcd->objectTypeIndication = GPAC_OTI_VIDEO_MPEG2_SPATIAL;
+					dcd->objectTypeIndication = GF_CODECID_MPEG2_SPATIAL;
 					ret = 1;
 				}
 				else if (!stricmp(val, "MPEG2VisualHigh")) {
-					dcd->objectTypeIndication = GPAC_OTI_VIDEO_MPEG2_HIGH;
+					dcd->objectTypeIndication = GF_CODECID_MPEG2_HIGH;
 					ret = 1;
 				}
 				else if (!stricmp(val, "MPEG2Visual422")) {
-					dcd->objectTypeIndication = GPAC_OTI_VIDEO_MPEG2_422;
+					dcd->objectTypeIndication = GF_CODECID_MPEG2_422;
 					ret = 1;
 				}
 				else if (!stricmp(val, "MPEG2AudioMain")) {
-					dcd->objectTypeIndication = GPAC_OTI_AUDIO_AAC_MPEG2_MP;
+					dcd->objectTypeIndication = GF_CODECID_AAC_MPEG2_MP;
 					ret = 1;
 				}
 				else if (!stricmp(val, "MPEG2AudioLowComplexity")) {
-					dcd->objectTypeIndication = GPAC_OTI_AUDIO_AAC_MPEG2_LCP;
+					dcd->objectTypeIndication = GF_CODECID_AAC_MPEG2_LCP;
 					ret = 1;
 				}
 				else if (!stricmp(val, "MPEG2AudioScaleableSamplingRate")) {
-					dcd->objectTypeIndication = GPAC_OTI_AUDIO_AAC_MPEG2_SSRP;
+					dcd->objectTypeIndication = GF_CODECID_AAC_MPEG2_SSRP;
 					ret = 1;
 				}
 				else if (!stricmp(val, "MPEG2AudioPart3")) {
-					dcd->objectTypeIndication = GPAC_OTI_AUDIO_MPEG2_PART3;
+					dcd->objectTypeIndication = GF_CODECID_MPEG2_PART3;
 					ret = 1;
 				}
 				else if (!stricmp(val, "MPEG1Visual")) {
-					dcd->objectTypeIndication = GPAC_OTI_VIDEO_MPEG1;
+					dcd->objectTypeIndication = GF_CODECID_MPEG1;
 					ret = 1;
 				}
 				else if (!stricmp(val, "MPEG1Audio")) {
-					dcd->objectTypeIndication = GPAC_OTI_AUDIO_MPEG1;
+					dcd->objectTypeIndication = GF_CODECID_MPEG_AUDIO;
 					ret = 1;
 				}
 				else if (!stricmp(val, "JPEG")) {
-					dcd->objectTypeIndication = GPAC_OTI_IMAGE_JPEG;
+					dcd->objectTypeIndication = GF_CODECID_JPEG;
 					ret = 1;
 				}
 				else if (!stricmp(val, "PNG")) {
-					dcd->objectTypeIndication = GPAC_OTI_IMAGE_PNG;
+					dcd->objectTypeIndication = GF_CODECID_PNG;
 					ret = 1;
 				}
 			}
@@ -359,48 +309,17 @@ GF_Err gf_odf_set_field(GF_Descriptor *desc, char *fieldName, char *val)
 			GET_U8(dcd->streamType)
 			/*XMT may use string*/
 			if (!ret) {
-				if (!stricmp(val, "ObjectDescriptor")) {
-					dcd->streamType = GF_STREAM_OD;
+				dcd->streamType = gf_stream_type_by_name(val);
+				if (dcd->streamType != GF_STREAM_UNKNOWN)
 					ret = 1;
-				}
-				else if (!stricmp(val, "ClockReference")) {
-					dcd->streamType = GF_STREAM_OCR;
-					ret = 1;
-				}
-				else if (!stricmp(val, "SceneDescription")) {
-					dcd->streamType = GF_STREAM_SCENE;
-					ret = 1;
-				}
-				else if (!stricmp(val, "Visual")) {
-					dcd->streamType = GF_STREAM_VISUAL;
-					ret = 1;
-				}
-				else if (!stricmp(val, "Audio")) {
-					dcd->streamType = GF_STREAM_AUDIO;
-					ret = 1;
-				}
-				else if (!stricmp(val, "MPEG7")) {
-					dcd->streamType = GF_STREAM_MPEG7;
-					ret = 1;
-				}
-				else if (!stricmp(val, "IPMP")) {
-					dcd->streamType = GF_STREAM_IPMP;
-					ret = 1;
-				}
-				else if (!stricmp(val, "OCI")) {
-					dcd->streamType = GF_STREAM_OCI;
-					ret = 1;
-				}
-				else if (!stricmp(val, "MPEGJ")) {
-					dcd->streamType = GF_STREAM_MPEGJ;
-					ret = 1;
-				}
 			}
 		}
-		else if (!stricmp(fieldName, "upStream")) GET_BOOL(dcd->upstream)
-			else if (!stricmp(fieldName, "bufferSizeDB")) ret += sscanf(val, "%u", &dcd->bufferSizeDB);
-			else if (!stricmp(fieldName, "maxBitRate")) ret += sscanf(val, "%u", &dcd->maxBitrate);
-			else if (!stricmp(fieldName, "avgBitRate")) ret += sscanf(val, "%u", &dcd->avgBitrate);
+		else if (!stricmp(fieldName, "upStream")) {
+			GET_BOOL(dcd->upstream)
+		}
+		else if (!stricmp(fieldName, "bufferSizeDB")) ret += sscanf(val, "%u", &dcd->bufferSizeDB);
+		else if (!stricmp(fieldName, "maxBitRate")) ret += sscanf(val, "%u", &dcd->maxBitrate);
+		else if (!stricmp(fieldName, "avgBitRate")) ret += sscanf(val, "%u", &dcd->avgBitrate);
 	}
 	break;
 	case GF_ODF_ESD_TAG:
@@ -410,53 +329,46 @@ GF_Err gf_odf_set_field(GF_Descriptor *desc, char *fieldName, char *val)
 			ret += sscanf(val, "%hu", &esd->ESID);
 		}
 		else if (!stricmp(fieldName, "streamPriority")) GET_U8(esd->streamPriority)
-			else if (!stricmp(fieldName, "dependsOn_ES_ID") || !stricmp(fieldName, "dependsOnESID")) ret += sscanf(val, "%hu", &esd->dependsOnESID);
-			else if (!stricmp(fieldName, "OCR_ES_ID")) ret += sscanf(val, "%hu", &esd->OCRESID);
-			else if (!stricmp(fieldName, "URLstring")) {
-				esd->URLString = gf_strdup(val);
-				ret = 1;
-			}
+		else if (!stricmp(fieldName, "dependsOn_ES_ID") || !stricmp(fieldName, "dependsOnESID")) ret += sscanf(val, "%hu", &esd->dependsOnESID);
+		else if (!stricmp(fieldName, "OCR_ES_ID")) ret += sscanf(val, "%hu", &esd->OCRESID);
+		else if (!stricmp(fieldName, "URLstring")) {
+			esd->URLString = gf_strdup(val);
+			ret = 1;
+		}
 		/*ignore*/
-			else if (!stricmp(fieldName, "streamDependenceFlag")
-			         || !stricmp(fieldName, "URL_Flag")
-			         || !stricmp(fieldName, "OCRstreamFlag")
-			        )
-				ret = 1;
+		else if (!stricmp(fieldName, "streamDependenceFlag")
+			 || !stricmp(fieldName, "URL_Flag")
+			 || !stricmp(fieldName, "OCRstreamFlag")
+		)
+			ret = 1;
 	}
 	break;
 	case GF_ODF_SLC_TAG:
 	{
-		u32 ts;
 		GF_SLConfig *slc = (GF_SLConfig*)desc;
 		if (!stricmp(fieldName, "predefined")) GET_U8(slc->predefined)
-			else if (!stricmp(fieldName, "useAccessUnitStartFlag")) GET_BOOL(slc->useAccessUnitStartFlag)
-				else if (!stricmp(fieldName, "useAccessUnitEndFlag")) GET_BOOL(slc->useAccessUnitEndFlag)
-					else if (!stricmp(fieldName, "useRandomAccessPointFlag")) GET_BOOL(slc->useRandomAccessPointFlag)
-						else if (!stricmp(fieldName, "hasRandomAccessUnitsOnlyFlag") || !stricmp(fieldName, "useRandomAccessUnitsOnlyFlag")) GET_BOOL(slc->hasRandomAccessUnitsOnlyFlag)
-							else if (!stricmp(fieldName, "usePaddingFlag")) GET_BOOL(slc->usePaddingFlag)
-								else if (!stricmp(fieldName, "useTimeStampsFlag")) GET_BOOL(slc->useTimestampsFlag)
-									else if (!stricmp(fieldName, "useIdleFlag")) GET_BOOL(slc->useIdleFlag)
-										else if (!stricmp(fieldName, "timeStampResolution")) ret += sscanf(val, "%u", &slc->timestampResolution);
-										else if (!stricmp(fieldName, "OCRResolution")) ret += sscanf(val, "%u", &slc->OCRResolution);
-										else if (!stricmp(fieldName, "timeStampLength")) GET_U8(slc->timestampLength)
-											else if (!stricmp(fieldName, "OCRLength")) GET_U8(slc->OCRLength)
-												else if (!stricmp(fieldName, "AU_Length")) GET_U8(slc->AULength)
-													else if (!stricmp(fieldName, "instantBitrateLength")) GET_U8(slc->instantBitrateLength)
-														else if (!stricmp(fieldName, "degradationPriorityLength")) GET_U8(slc->degradationPriorityLength)
-															else if (!stricmp(fieldName, "AU_seqNumLength")) GET_U8(slc->AUSeqNumLength)
-																else if (!stricmp(fieldName, "packetSeqNumLength")) GET_U8(slc->packetSeqNumLength)
-																	else if (!stricmp(fieldName, "timeScale")) ret += sscanf(val, "%u", &slc->timeScale);
-																	else if (!stricmp(fieldName, "accessUnitDuration")) ret += sscanf(val, "%hu", &slc->AUDuration);
-																	else if (!stricmp(fieldName, "compositionUnitDuration")) ret += sscanf(val, "%hu", &slc->CUDuration);
-																	else if (!stricmp(fieldName, "startDecodingTimeStamp")) {
-																		ret += sscanf(val, "%u", &ts);
-																		slc->startDTS = ts;
-																	}
-																	else if (!stricmp(fieldName, "startCompositionTimeStamp")) {
-																		ret += sscanf(val, "%u", &ts);
-																		slc->startCTS = ts;
-																	}
-																	else if (!stricmp(fieldName, "durationFlag")) ret = 1;
+		else if (!stricmp(fieldName, "useAccessUnitStartFlag")) GET_BOOL(slc->useAccessUnitStartFlag)
+		else if (!stricmp(fieldName, "useAccessUnitEndFlag")) GET_BOOL(slc->useAccessUnitEndFlag)
+		else if (!stricmp(fieldName, "useRandomAccessPointFlag")) GET_BOOL(slc->useRandomAccessPointFlag)
+		else if (!stricmp(fieldName, "hasRandomAccessUnitsOnlyFlag") || !stricmp(fieldName, "useRandomAccessUnitsOnlyFlag")) GET_BOOL(slc->hasRandomAccessUnitsOnlyFlag)
+		else if (!stricmp(fieldName, "usePaddingFlag")) GET_BOOL(slc->usePaddingFlag)
+		else if (!stricmp(fieldName, "useTimeStampsFlag")) GET_BOOL(slc->useTimestampsFlag)
+		else if (!stricmp(fieldName, "useIdleFlag")) GET_BOOL(slc->useIdleFlag)
+		else if (!stricmp(fieldName, "timeStampResolution")) ret += sscanf(val, "%u", &slc->timestampResolution);
+		else if (!stricmp(fieldName, "OCRResolution")) ret += sscanf(val, "%u", &slc->OCRResolution);
+		else if (!stricmp(fieldName, "timeStampLength")) GET_U8(slc->timestampLength)
+		else if (!stricmp(fieldName, "OCRLength")) GET_U8(slc->OCRLength)
+		else if (!stricmp(fieldName, "AU_Length")) GET_U8(slc->AULength)
+		else if (!stricmp(fieldName, "instantBitrateLength")) GET_U8(slc->instantBitrateLength)
+		else if (!stricmp(fieldName, "degradationPriorityLength")) GET_U8(slc->degradationPriorityLength)
+		else if (!stricmp(fieldName, "AU_seqNumLength")) GET_U8(slc->AUSeqNumLength)
+		else if (!stricmp(fieldName, "packetSeqNumLength")) GET_U8(slc->packetSeqNumLength)
+		else if (!stricmp(fieldName, "timeScale")) ret += sscanf(val, "%u", &slc->timeScale);
+		else if (!stricmp(fieldName, "accessUnitDuration")) ret += sscanf(val, "%hu", &slc->AUDuration);
+		else if (!stricmp(fieldName, "compositionUnitDuration")) ret += sscanf(val, "%hu", &slc->CUDuration);
+		else if (!stricmp(fieldName, "startDecodingTimeStamp")) GET_U64(slc->startDTS)
+		else if (!stricmp(fieldName, "startCompositionTimeStamp"))  GET_U64(slc->startCTS)
+		else if (!stricmp(fieldName, "durationFlag")) ret = 1;
 	}
 	break;
 	case GF_ODF_ELEM_MASK_TAG:
@@ -481,57 +393,54 @@ GF_Err gf_odf_set_field(GF_Descriptor *desc, char *fieldName, char *val)
 			/*GET_BOOL(bcd->isCommandStream)*/ ret = 1;
 		}
 		else if (!stricmp(fieldName, "pixelMetric") || !stricmp(fieldName, "pixelMetrics")) GET_BOOL(bcd->pixelMetrics)
-			else if (!stricmp(fieldName, "pixelWidth")) ret += sscanf(val, "%hu", &bcd->pixelWidth);
-			else if (!stricmp(fieldName, "pixelHeight")) ret += sscanf(val, "%hu", &bcd->pixelHeight);
-			else if (!stricmp(fieldName, "use3DMeshCoding")) ret = 1;
-			else if (!stricmp(fieldName, "usePredictiveMFField")) ret = 1;
-			else if (!stricmp(fieldName, "randomAccess")) GET_BOOL(bcd->randomAccess)
-				else if (!stricmp(fieldName, "useNames")) GET_BOOL(bcd->useNames)
-				}
+		else if (!stricmp(fieldName, "pixelWidth")) ret += sscanf(val, "%hu", &bcd->pixelWidth);
+		else if (!stricmp(fieldName, "pixelHeight")) ret += sscanf(val, "%hu", &bcd->pixelHeight);
+		else if (!stricmp(fieldName, "use3DMeshCoding")) ret = 1;
+		else if (!stricmp(fieldName, "usePredictiveMFField")) ret = 1;
+		else if (!stricmp(fieldName, "randomAccess")) GET_BOOL(bcd->randomAccess)
+		else if (!stricmp(fieldName, "useNames")) GET_BOOL(bcd->useNames)
+	}
 	break;
 	case GF_ODF_MUXINFO_TAG:
 	{
 		GF_MuxInfo *mi = (GF_MuxInfo *)desc;
 		if (!stricmp(fieldName, "fileName") || !stricmp(fieldName, "url")) GET_STRING(mi->file_name)
-			else if (!stricmp(fieldName, "streamFormat")) GET_STRING(mi->streamFormat)
-				else if (!stricmp(fieldName, "GroupID")) ret += sscanf(val, "%u", &mi->GroupID);
-				else if (!stricmp(fieldName, "startTime")) ret += sscanf(val, "%d", &mi->startTime);
-				else if (!stricmp(fieldName, "duration")) ret += sscanf(val, "%u", &mi->duration);
-				else if (!stricmp(fieldName, "carouselPeriod")) {
-					ret += sscanf(val, "%u", &mi->carousel_period_plus_one);
-					mi->carousel_period_plus_one += 1;
-				}
-				else if (!stricmp(fieldName, "aggregateOnESID")) ret += sscanf(val, "%hu", &mi->aggregate_on_esid);
+		else if (!stricmp(fieldName, "streamFormat")) GET_STRING(mi->streamFormat)
+		else if (!stricmp(fieldName, "GroupID")) ret += sscanf(val, "%u", &mi->GroupID);
+		else if (!stricmp(fieldName, "startTime")) ret += sscanf(val, "%d", &mi->startTime);
+		else if (!stricmp(fieldName, "duration")) ret += sscanf(val, "%u", &mi->duration);
+		else if (!stricmp(fieldName, "carouselPeriod")) {
+			ret += sscanf(val, "%u", &mi->carousel_period_plus_one);
+			mi->carousel_period_plus_one += 1;
+		}
+		else if (!stricmp(fieldName, "aggregateOnESID")) ret += sscanf(val, "%hu", &mi->aggregate_on_esid);
 
 #ifndef GPAC_DISABLE_MEDIA_IMPORT
-				else if (!stricmp(fieldName, "compactSize"))
-				{
-					ret = 1;
-					if (!stricmp(val, "true") || !stricmp(val, "1")) mi->import_flags |= GF_IMPORT_USE_COMPACT_SIZE;
-				}
-				else if (!stricmp(fieldName, "useDataReference"))
-				{
-					ret = 1;
-					if (!stricmp(val, "true") || !stricmp(val, "1")) mi->import_flags |= GF_IMPORT_USE_DATAREF;
-				}
-				else if (!stricmp(fieldName, "noFrameDrop"))
-				{
-					ret = 1;
-					if (!stricmp(val, "true") || !stricmp(val, "1")) mi->import_flags |= GF_IMPORT_NO_FRAME_DROP;
-				}
-				else if (!stricmp(fieldName, "SBR_Type")) {
-					ret = 1;
-					if (!stricmp(val, "implicit") || !stricmp(val, "1")) mi->import_flags |= GF_IMPORT_SBR_IMPLICIT;
-					else if (!stricmp(val, "explicit") || !stricmp(val, "2")) mi->import_flags |= GF_IMPORT_SBR_EXPLICIT;
-				}
+		else if (!stricmp(fieldName, "compactSize"))
+		{
+			ret = 1;
+			if (!stricmp(val, "true") || !stricmp(val, "1")) mi->import_flags |= GF_IMPORT_USE_COMPACT_SIZE;
+		}
+		else if (!stricmp(fieldName, "useDataReference")) {
+			ret = 1;
+			if (!stricmp(val, "true") || !stricmp(val, "1")) mi->import_flags |= GF_IMPORT_USE_DATAREF;
+		}
+		else if (!stricmp(fieldName, "noFrameDrop")) {
+			ret = 1;
+			if (!stricmp(val, "true") || !stricmp(val, "1")) mi->import_flags |= GF_IMPORT_NO_FRAME_DROP;
+		}
+		else if (!stricmp(fieldName, "SBR_Type")) {
+			ret = 1;
+			if (!stricmp(val, "implicit") || !stricmp(val, "1")) mi->import_flags |= GF_IMPORT_SBR_IMPLICIT;
+			else if (!stricmp(val, "explicit") || !stricmp(val, "2")) mi->import_flags |= GF_IMPORT_SBR_EXPLICIT;
+		}
 #endif /*GPAC_DISABLE_MEDIA_IMPORT*/
-
-				else if (!stricmp(fieldName, "textNode")) GET_STRING(mi->textNode)
-					else if (!stricmp(fieldName, "fontNode")) GET_STRING(mi->fontNode)
-						else if (!stricmp(fieldName, "frameRate")) {
-							ret = 1;
-							mi->frame_rate = atof(val);
-						}
+		else if (!stricmp(fieldName, "textNode")) GET_STRING(mi->textNode)
+		else if (!stricmp(fieldName, "fontNode")) GET_STRING(mi->fontNode)
+		else if (!stricmp(fieldName, "frameRate")) {
+			ret = 1;
+			mi->frame_rate = atof(val);
+		}
 	}
 	break;
 	case GF_ODF_DSI_TAG:
@@ -543,8 +452,8 @@ GF_Err gf_odf_set_field(GF_Descriptor *desc, char *fieldName, char *val)
 				OD_ParseBinData(val, &dsi->data, &dsi->dataLength);
 				ret = 1;
 			} else if (!strnicmp(val, "file:", 5)) {
-				OD_ParseFileData(val+5, &dsi->data, &dsi->dataLength);
-				ret = 1;
+				if (gf_file_load_data(val+5, (u8 **) &dsi->data, &dsi->dataLength) == GF_OK)
+					ret = 1;
 			} else if (!strlen(val)) ret = 1;
 		}
 		if (!stricmp(fieldName, "src")) {
@@ -556,8 +465,8 @@ GF_Err gf_odf_set_field(GF_Descriptor *desc, char *fieldName, char *val)
 				OD_ParseBinData(val, &dsi->data, &dsi->dataLength);
 				ret = 1;
 			} else if (!strnicmp(val, "file:", 5)) {
-				OD_ParseFileData(val+5, &dsi->data, &dsi->dataLength);
-				ret = 1;
+				if (gf_file_load_data(val+5, (u8 **) &dsi->data, &dsi->dataLength) == GF_OK)
+					ret = 1;
 			}
 		}
 	}
@@ -566,28 +475,32 @@ GF_Err gf_odf_set_field(GF_Descriptor *desc, char *fieldName, char *val)
 	{
 		GF_Segment *sd = (GF_Segment*)desc;
 		if (!stricmp(fieldName, "start") || !stricmp(fieldName, "startTime")) GET_DOUBLE(sd->startTime)
-			else if (!stricmp(fieldName, "duration")) GET_DOUBLE(sd->Duration)
-				else if (!stricmp(fieldName, "name") || !stricmp(fieldName, "segmentName")) GET_STRING(sd->SegmentName)
-				}
+		else if (!stricmp(fieldName, "duration")) GET_DOUBLE(sd->Duration)
+		else if (!stricmp(fieldName, "name") || !stricmp(fieldName, "segmentName")) GET_STRING(sd->SegmentName)
+	}
 	break;
 	case GF_ODF_UI_CFG_TAG:
 	{
 		GF_UIConfig *uic = (GF_UIConfig*)desc;
 		if (!stricmp(fieldName, "deviceName")) GET_STRING(uic->deviceName)
-			else if (!stricmp(fieldName, "termChar")) GET_U8(uic->termChar)
-				else if (!stricmp(fieldName, "delChar")) GET_U8(uic->delChar)
-					else if (!stricmp(fieldName, "uiData")) {
-						/*only parse true hexa strings*/
-						if (val[0] == '%') {
-							OD_ParseBinData(val, &uic->ui_data, &uic->ui_data_length);
-							ret = 1;
-						} else if (!strnicmp(val, "file:", 5)) {
-							OD_ParseFileData(val+5, &uic->ui_data, &uic->ui_data_length);
-							ret = 1;
-						} else {
-							ret = OD_ParseUIConfig(val, &uic->ui_data, &uic->ui_data_length);
-						}
-					}
+		else if (!stricmp(fieldName, "termChar")) GET_U8(uic->termChar)
+		else if (!stricmp(fieldName, "delChar")) GET_U8(uic->delChar)
+		else if (!stricmp(fieldName, "uiData")) {
+			/*only parse true hexa strings*/
+			if (val[0] == '%') {
+				OD_ParseBinData(val, &uic->ui_data, &uic->ui_data_length);
+				ret = 1;
+			} else if (!strnicmp(val, "file:", 5)) {
+				if (gf_file_load_data(val+5, (u8 **) &uic->ui_data, &uic->ui_data_length)==GF_OK) 
+					ret = 1;
+			} else {
+#ifndef GPAC_MINIMAL_ODF
+				ret = OD_ParseUIConfig(val, &uic->ui_data, &uic->ui_data_length);
+#else
+				ret= GF_FALSE;
+#endif
+			}
+		}
 	}
 	break;
 	case GF_ODF_ESD_INC_TAG:
@@ -606,100 +519,100 @@ GF_Err gf_odf_set_field(GF_Descriptor *desc, char *fieldName, char *val)
 	{
 		GF_TextConfig *txt = (GF_TextConfig*)desc;
 		if (!stricmp(fieldName, "3GPPBaseFormat")) GET_U8(txt->Base3GPPFormat)
-			else if (!stricmp(fieldName, "MPEGExtendedFormat")) GET_U8(txt->MPEGExtendedFormat)
-				else if (!stricmp(fieldName, "profileLevel")) GET_U8(txt->profileLevel)
-					else if (!stricmp(fieldName, "durationClock") || !stricmp(fieldName, "timescale") ) GET_U32(txt->timescale)
-						else if (!stricmp(fieldName, "layer")) GET_U32(txt->layer)
-							else if (!stricmp(fieldName, "text_width")) GET_U32(txt->text_width)
-								else if (!stricmp(fieldName, "text_height")) GET_U32(txt->text_height)
-									else if (!stricmp(fieldName, "video_width")) {
-										GET_U32(txt->video_width)
-										txt->has_vid_info = GF_TRUE;
-									}
-									else if (!stricmp(fieldName, "video_height")) {
-										GET_U32(txt->video_height)
-										txt->has_vid_info = GF_TRUE;
-									}
-									else if (!stricmp(fieldName, "horizontal_offset")) {
-										GET_S16(txt->horiz_offset)
-										txt->has_vid_info = GF_TRUE;
-									}
-									else if (!stricmp(fieldName, "vertical_offset")) {
-										GET_S32(txt->vert_offset)
-										txt->has_vid_info = GF_TRUE;
-									}
+		else if (!stricmp(fieldName, "MPEGExtendedFormat")) GET_U8(txt->MPEGExtendedFormat)
+		else if (!stricmp(fieldName, "profileLevel")) GET_U8(txt->profileLevel)
+		else if (!stricmp(fieldName, "durationClock") || !stricmp(fieldName, "timescale") ) GET_U32(txt->timescale)
+		else if (!stricmp(fieldName, "layer")) GET_U32(txt->layer)
+		else if (!stricmp(fieldName, "text_width")) GET_U32(txt->text_width)
+		else if (!stricmp(fieldName, "text_height")) GET_U32(txt->text_height)
+		else if (!stricmp(fieldName, "video_width")) {
+			GET_U32(txt->video_width)
+			txt->has_vid_info = GF_TRUE;
+		}
+		else if (!stricmp(fieldName, "video_height")) {
+			GET_U32(txt->video_height)
+			txt->has_vid_info = GF_TRUE;
+		}
+		else if (!stricmp(fieldName, "horizontal_offset")) {
+			GET_S16(txt->horiz_offset)
+			txt->has_vid_info = GF_TRUE;
+		}
+		else if (!stricmp(fieldName, "vertical_offset")) {
+			GET_S32(txt->vert_offset)
+			txt->has_vid_info = GF_TRUE;
+		}
 	}
 	break;
 	case GF_ODF_TX3G_TAG:
 	{
 		GF_TextSampleDescriptor *sd = (GF_TextSampleDescriptor*)desc;
 		if (!stricmp(fieldName, "displayFlags")) GET_U32(sd->displayFlags)
-			else if (!stricmp(fieldName, "horiz_justif")) GET_S32(sd->horiz_justif)
-				else if (!stricmp(fieldName, "vert_justif")) GET_S32(sd->vert_justif)
-					else if (!stricmp(fieldName, "back_color")) GET_S32(sd->back_color)
-						else if (!stricmp(fieldName, "top")) GET_S32(sd->default_pos.top)
-							else if (!stricmp(fieldName, "bottom")) GET_S32(sd->default_pos.bottom)
-								else if (!stricmp(fieldName, "left")) GET_S32(sd->default_pos.left)
-									else if (!stricmp(fieldName, "right")) GET_S32(sd->default_pos.right)
-										else if (!stricmp(fieldName, "style_font_ID")) GET_S32(sd->default_style.fontID)
-											else if (!stricmp(fieldName, "style_font_size")) GET_S32(sd->default_style.font_size)
-												else if (!stricmp(fieldName, "style_text_color")) GET_U32(sd->default_style.text_color)
-													else if (!stricmp(fieldName, "style_flags")) {
-														char szStyles[1024];
-														strcpy(szStyles, val);
-														strlwr(szStyles);
-														if (strstr(szStyles, "bold")) sd->default_style.style_flags |= GF_TXT_STYLE_BOLD;
-														if (strstr(szStyles, "italic")) sd->default_style.style_flags |= GF_TXT_STYLE_ITALIC;
-														if (strstr(szStyles, "underlined")) sd->default_style.style_flags |= GF_TXT_STYLE_UNDERLINED;
-														ret = 1;
-													}
-													else if (!stricmp(fieldName, "fontID") || !stricmp(fieldName, "fontName")) {
-														/*check if we need a new entry*/
-														if (!sd->font_count) {
-															sd->fonts = (GF_FontRecord*)gf_malloc(sizeof(GF_FontRecord));
-															sd->font_count = 1;
-															sd->fonts[0].fontID = 0;
-															sd->fonts[0].fontName = NULL;
-														} else {
-															Bool realloc_fonts = GF_FALSE;
-															if (!stricmp(fieldName, "fontID") && sd->fonts[sd->font_count-1].fontID) realloc_fonts = GF_TRUE;
-															else if (!stricmp(fieldName, "fontName") && sd->fonts[sd->font_count-1].fontName) realloc_fonts = GF_TRUE;
-															if (realloc_fonts) {
-																sd->font_count += 1;
-																sd->fonts = (GF_FontRecord*)gf_realloc(sd->fonts, sizeof(GF_FontRecord)*sd->font_count);
-																sd->fonts[sd->font_count-1].fontID = 0;
-																sd->fonts[sd->font_count-1].fontName = NULL;
-															}
-														}
-														if (!stricmp(fieldName, "fontID")) GET_U32(sd->fonts[sd->font_count-1].fontID)
-															if (!stricmp(fieldName, "fontName")) GET_STRING(sd->fonts[sd->font_count-1].fontName)
-																ret = 1;
-													}
+		else if (!stricmp(fieldName, "horiz_justif")) GET_S32(sd->horiz_justif)
+		else if (!stricmp(fieldName, "vert_justif")) GET_S32(sd->vert_justif)
+		else if (!stricmp(fieldName, "back_color")) GET_S32(sd->back_color)
+		else if (!stricmp(fieldName, "top")) GET_S32(sd->default_pos.top)
+		else if (!stricmp(fieldName, "bottom")) GET_S32(sd->default_pos.bottom)
+		else if (!stricmp(fieldName, "left")) GET_S32(sd->default_pos.left)
+		else if (!stricmp(fieldName, "right")) GET_S32(sd->default_pos.right)
+		else if (!stricmp(fieldName, "style_font_ID")) GET_S32(sd->default_style.fontID)
+		else if (!stricmp(fieldName, "style_font_size")) GET_S32(sd->default_style.font_size)
+		else if (!stricmp(fieldName, "style_text_color")) GET_U32(sd->default_style.text_color)
+		else if (!stricmp(fieldName, "style_flags")) {
+			char szStyles[1024];
+			strcpy(szStyles, val);
+			strlwr(szStyles);
+			if (strstr(szStyles, "bold")) sd->default_style.style_flags |= GF_TXT_STYLE_BOLD;
+			if (strstr(szStyles, "italic")) sd->default_style.style_flags |= GF_TXT_STYLE_ITALIC;
+			if (strstr(szStyles, "underlined")) sd->default_style.style_flags |= GF_TXT_STYLE_UNDERLINED;
+			ret = 1;
+		}
+		else if (!stricmp(fieldName, "fontID") || !stricmp(fieldName, "fontName")) {
+			/*check if we need a new entry*/
+			if (!sd->font_count) {
+				sd->fonts = (GF_FontRecord*)gf_malloc(sizeof(GF_FontRecord));
+				sd->font_count = 1;
+				sd->fonts[0].fontID = 0;
+				sd->fonts[0].fontName = NULL;
+			} else {
+				Bool realloc_fonts = GF_FALSE;
+				if (!stricmp(fieldName, "fontID") && sd->fonts[sd->font_count-1].fontID) realloc_fonts = GF_TRUE;
+				else if (!stricmp(fieldName, "fontName") && sd->fonts[sd->font_count-1].fontName) realloc_fonts = GF_TRUE;
+				if (realloc_fonts) {
+					sd->font_count += 1;
+					sd->fonts = (GF_FontRecord*)gf_realloc(sd->fonts, sizeof(GF_FontRecord)*sd->font_count);
+					sd->fonts[sd->font_count-1].fontID = 0;
+					sd->fonts[sd->font_count-1].fontName = NULL;
+				}
+			}
+			if (!stricmp(fieldName, "fontID")) GET_U32(sd->fonts[sd->font_count-1].fontID)
+			if (!stricmp(fieldName, "fontName")) GET_STRING(sd->fonts[sd->font_count-1].fontName)
+		}
 	}
 	break;
 	case GF_ODF_IPMP_TAG:
 	{
 		GF_IPMP_Descriptor *ipmp = (GF_IPMP_Descriptor*)desc;
 		if (!stricmp(fieldName, "IPMP_DescriptorID")) GET_U8(ipmp->IPMP_DescriptorID)
-			else if (!stricmp(fieldName, "IPMPS_Type")) GET_U16(ipmp->IPMPS_Type)
-				else if (!stricmp(fieldName, "IPMP_DescriptorIDEx")) GET_U16(ipmp->IPMP_DescriptorIDEx)
-					else if (!stricmp(fieldName, "IPMP_ToolID")) {
-						ret = 1;
-						OD_ParseBin128(val, &ipmp->IPMP_ToolID);
-					}
-					else if (!stricmp(fieldName, "controlPointCode")) GET_U8(ipmp->control_point)
-						else if (!stricmp(fieldName, "sequenceCode")) GET_U8(ipmp->cp_sequence_code)
-						}
+		else if (!stricmp(fieldName, "IPMPS_Type")) GET_U16(ipmp->IPMPS_Type)
+		else if (!stricmp(fieldName, "IPMP_DescriptorIDEx")) GET_U16(ipmp->IPMP_DescriptorIDEx)
+		else if (!stricmp(fieldName, "IPMP_ToolID")) {
+			ret = 1;
+			gf_bin128_parse(val, ipmp->IPMP_ToolID);
+		}
+		else if (!stricmp(fieldName, "controlPointCode")) GET_U8(ipmp->control_point)
+		else if (!stricmp(fieldName, "sequenceCode")) GET_U8(ipmp->cp_sequence_code)
+	}
 	break;
 	case GF_ODF_IPMP_PTR_TAG:
 	{
 		GF_IPMPPtr *ipmpd = (GF_IPMPPtr*)desc;
 		if (!stricmp(fieldName, "IPMP_DescriptorID")) GET_U8(ipmpd->IPMP_DescriptorID)
-			else if (!stricmp(fieldName, "IPMP_DescriptorIDEx"))  ret += sscanf(val, "%hu", &ipmpd->IPMP_DescriptorIDEx);
-			else if (!stricmp(fieldName, "IPMP_ES_ID"))  ret += sscanf(val, "%hu", &ipmpd->IPMP_ES_ID);
+		else if (!stricmp(fieldName, "IPMP_DescriptorIDEx"))  ret += sscanf(val, "%hu", &ipmpd->IPMP_DescriptorIDEx);
+		else if (!stricmp(fieldName, "IPMP_ES_ID"))  ret += sscanf(val, "%hu", &ipmpd->IPMP_ES_ID);
 	}
 	break;
 	case GF_ODF_LANG_TAG:
+	case GF_ODF_GPAC_LANG:
 	{
 		GF_Language *ld = (GF_Language *)desc;
 		if (!stricmp(fieldName, "languageCode")) {
@@ -717,22 +630,22 @@ GF_Err gf_odf_set_field(GF_Descriptor *desc, char *fieldName, char *val)
 	{
 		GF_AuxVideoDescriptor *avd = (GF_AuxVideoDescriptor *)desc;
 		if (!stricmp(fieldName, "aux_video_type"))  GET_U8(avd->aux_video_type)
-			if (!stricmp(fieldName, "position_offset_h"))  GET_U8(avd->aux_video_type)
-				if (!stricmp(fieldName, "position_offset_v"))  GET_U8(avd->aux_video_type)
-					if (!stricmp(fieldName, "knear"))  GET_U8(avd->aux_video_type)
-						if (!stricmp(fieldName, "kfar"))  GET_U8(avd->aux_video_type)
-							if (!stricmp(fieldName, "parallax_zero"))  GET_U16(avd->aux_video_type)
-								if (!stricmp(fieldName, "parallax_scale"))  GET_U16(avd->aux_video_type)
-									if (!stricmp(fieldName, "dref"))  GET_U16(avd->aux_video_type)
-										if (!stricmp(fieldName, "wref"))  GET_U16(avd->aux_video_type)
-										}
+		else if (!stricmp(fieldName, "position_offset_h"))  GET_U8(avd->position_offset_h)
+		else if (!stricmp(fieldName, "position_offset_v"))  GET_U8(avd->position_offset_v)
+		else if (!stricmp(fieldName, "knear"))  GET_U8(avd->knear)
+		else if (!stricmp(fieldName, "kfar"))  GET_U8(avd->kfar)
+		else if (!stricmp(fieldName, "parallax_zero"))  GET_U16(avd->parallax_zero)
+		else if (!stricmp(fieldName, "parallax_scale"))  GET_U16(avd->parallax_scale)
+		else if (!stricmp(fieldName, "dref"))  GET_U16(avd->dref)
+		else if (!stricmp(fieldName, "wref"))  GET_U16(avd->wref)
+	}
 	break;
 	case GF_ODF_IPMP_TOOL_TAG:
 	{
 		GF_IPMP_Tool *it = (GF_IPMP_Tool*)desc;
 		if (!stricmp(fieldName, "IPMP_ToolID")) {
 			ret = 1;
-			OD_ParseBin128(val, &it->IPMP_ToolID);
+			gf_bin128_parse(val, it->IPMP_ToolID);
 		}
 		else if (!stricmp(fieldName, "ToolURL"))  GET_STRING(it->tool_url)
 		}
@@ -742,9 +655,9 @@ GF_Err gf_odf_set_field(GF_Descriptor *desc, char *fieldName, char *val)
 	return ret ? GF_OK : GF_BAD_PARAM;
 }
 
-Bool OD_ParseUIConfig(char *val, char **out_data, u32 *out_data_size)
-{
 #ifndef GPAC_MINIMAL_ODF
+Bool OD_ParseUIConfig(u8 *val, u8 **out_data, u32 *out_data_size)
+{
 	GF_BitStream *bs;
 	if (!strnicmp(val, "HTK:", 4)) {
 		char szItem[100];
@@ -809,6 +722,6 @@ Bool OD_ParseUIConfig(char *val, char **out_data, u32 *out_data_size)
 		gf_bs_del(bs);
 		return GF_TRUE;
 	}
-#endif
 	return GF_FALSE;
 }
+#endif
