@@ -28,7 +28,7 @@
 
 #ifndef GPAC_DISABLE_SVG
 
-static void gf_smil_timing_null_timed_function(SMIL_Timing_RTI *rti, Fixed normalized_scene_time, u32 state)
+static void gf_smil_timing_null_timed_function(SMIL_Timing_RTI *rti, Fixed normalized_scene_time, GF_SGSMILTimingEvalState state)
 {
 }
 
@@ -224,7 +224,7 @@ static void gf_smil_timing_get_first_interval(SMIL_Timing_RTI *rti)
 
 	/* this is the first time we check the interval */
 	gf_smil_timing_get_interval_end(rti, rti->current_interval);
-	if (0 && rti->current_interval->end == -2) {
+	if ((0) && rti->current_interval->end == -2) {
 		/* TODO: check if the interval can be discarded (i.e. if end is specified with an invalid end value (return -2)),
 		   probably yes, but next time we call the evaluation of interval, we should call get_first_interval */
 		GF_LOG(GF_LOG_DEBUG, GF_LOG_SMIL, ("[SMIL Timing   ] Time %f - Timed element %s - Wrong Interval\n", gf_node_get_scene_time((GF_Node *)rti->timed_elt), gf_node_get_log_name((GF_Node *)rti->timed_elt)));
@@ -319,7 +319,7 @@ void gf_smil_timing_init_runtime_info(GF_Node *timed_elt)
 {
 	GF_SceneGraph *sg;
 	SMIL_Timing_RTI *rti;
-	SMILTimingAttributesPointers *timingp = NULL;
+	SMILTimingAttributesPointers *timingp;
 	u32 tag = gf_node_get_tag(timed_elt);
 	SVGAllAttributes all_atts;
 	SVGTimedAnimBaseElement *e = (SVGTimedAnimBaseElement *)timed_elt;
@@ -440,18 +440,11 @@ void gf_smil_timing_delete_runtime_info(GF_Node *timed_elt, SMIL_Timing_RTI *rti
 GF_EXPORT
 Bool gf_smil_timing_is_active(GF_Node *node)
 {
-	SMILTimingAttributesPointers *timingp = NULL;
-	timingp = ((SVGTimedAnimBaseElement *)node)->timingp;
+	SMILTimingAttributesPointers *timingp = ((SVGTimedAnimBaseElement *)node)->timingp;
 	if (!timingp || !timingp->runtime) return 0;
 	return (timingp->runtime->status == SMIL_STATUS_ACTIVE);
 }
 
-/* This function notifies the scene time to all the timed elements from the list in the given scene graph.
-   It returns the number of active timed elements. If no timed element is active, this means that from the timing
-   point of view, the scene has not changed and no rendering refresh is needed, even if the time has changed.
-   It uses an additional list of modified timed elements to insure that no timing
-   element was modified by the begin/end/repeat of another timed element.
-*/
 Bool gf_smil_notify_timed_elements(GF_SceneGraph *sg)
 {
 	SMIL_Timing_RTI *rti;
@@ -908,10 +901,8 @@ end:
 /* This function is called when a modification to the node has been made (scripts, updates or events ...) */
 void gf_smil_timing_modified(GF_Node *node, GF_FieldInfo *field)
 {
-	SMILTimingAttributesPointers *timingp = NULL;
 	SMIL_Timing_RTI *rti;
-
-	timingp = ((SVGTimedAnimBaseElement *)node)->timingp;
+	SMILTimingAttributesPointers *timingp = ((SVGTimedAnimBaseElement *)node)->timingp;
 
 	if (!timingp) return;
 	rti = timingp->runtime;
@@ -923,7 +914,7 @@ void gf_smil_timing_modified(GF_Node *node, GF_FieldInfo *field)
 	} else {
 		/* we don't have the right to modify the end of an element if it's not in unresolved state */
 		if (rti->current_interval->end == -1) gf_smil_timing_get_interval_end(rti, rti->current_interval);
-		if (0 && rti->current_interval->end == -2) {
+		if ((0) && rti->current_interval->end == -2) {
 			/* TODO: check if the interval can be discarded if end = -2,
 			   probably no, because the interval is currently running*/
 			GF_LOG(GF_LOG_DEBUG, GF_LOG_SMIL, ("[SMIL Timing   ] Time %f - Timed element %s - Wrong Interval\n", gf_node_get_scene_time((GF_Node *)rti->timed_elt), gf_node_get_log_name((GF_Node *)rti->timed_elt)));
@@ -992,10 +983,6 @@ Bool gf_svg_resolve_smil_times(GF_Node *anim, void *event_base_element,
 	return 1;
 }
 
-/* Inserts a new resolved time instant in the begin or end attribute.
-   The insertion preserves the sorting and removes the previous insertions
-    which have become obsolete
-   WARNING: Only used for inserting time when an <a> element, whose target is a timed element, is activated. */
 GF_EXPORT
 void gf_smil_timing_insert_clock(GF_Node *elt, Bool is_end, Double clock)
 {
@@ -1056,8 +1043,7 @@ void gf_smil_timing_resume(GF_Node *node)
 	}
 }
 
-void gf_smil_set_evaluation_callback(GF_Node *node,
-                                     void (*smil_evaluate)(struct _smil_timing_rti *rti, Fixed normalized_simple_time, u32 state))
+void gf_smil_set_evaluation_callback(GF_Node *node, gf_sg_smil_evaluate smil_evaluate)
 {
 	if (node && ((SVGTimedAnimBaseElement *)node)->timingp  && ((SVGTimedAnimBaseElement *)node)->timingp->runtime) {
 		SMIL_Timing_RTI *rti = ((SVGTimedAnimBaseElement *)node)->timingp->runtime;
@@ -1072,7 +1058,7 @@ GF_Node *gf_smil_get_element(SMIL_Timing_RTI *rti)
 
 Double gf_smil_get_media_duration(SMIL_Timing_RTI *rti)
 {
-	return rti->media_duration;
+	return rti ? rti->media_duration : 0.0;
 }
 
 
