@@ -2,7 +2,7 @@
  *			GPAC - Multimedia Framework C SDK
  *
  *			Authors: Jean Le Feuvre - Cyril Concolato
- *			Copyright (c) Telecom ParisTech 2000-2012
+ *			Copyright (c) Telecom ParisTech 2000-2020
  *					All rights reserved
  *
  *  This file is part of GPAC / SVG Scene Graph sub-project
@@ -46,6 +46,33 @@ enum
 	GF_SVG_ATTOPT_FILTER = 5,
 } GF_SVGAttOption;
 
+/* unused XBL tags (deprecated)
+
+	TAG_XBL_ATT_RANGE_FIRST = TAG_SVG_ATT_RANGE_FIRST + 256,
+	TAG_XBL_ATT_id = TAG_XBL_ATT_RANGE_FIRST,
+	TAG_XBL_ATT_extends,
+	TAG_XBL_ATT_display,
+	TAG_XBL_ATT_inheritstyle,
+	TAG_XBL_ATT_includes,
+	TAG_XBL_ATT_name,
+	TAG_XBL_ATT_implements,
+	TAG_XBL_ATT_type,
+	TAG_XBL_ATT_readonly,
+	TAG_XBL_ATT_onget,
+	TAG_XBL_ATT_onset,
+	TAG_XBL_ATT_event,
+	TAG_XBL_ATT_action,
+	TAG_XBL_ATT_phase,
+	TAG_XBL_ATT_button,
+	TAG_XBL_ATT_modifiers,
+	TAG_XBL_ATT_keycode,
+	TAG_XBL_ATT_key,
+	TAG_XBL_ATT_charcode,
+	TAG_XBL_ATT_clickcount,
+	TAG_XBL_ATT_command,
+	TAG_XBL_ATT_preventdefault,
+	TAG_XBL_ATT_src,
+*/
 static const struct xml_att_def {
 	const char *name;
 	u32 tag;
@@ -230,8 +257,8 @@ static const struct xml_att_def {
 	{ "fx", TAG_SVG_ATT_fx, SVG_Coordinate_datatype, 0, GF_XMLNS_SVG },
 	{ "fy", TAG_SVG_ATT_fy, SVG_Coordinate_datatype, 0, GF_XMLNS_SVG },
 	{ "size", TAG_SVG_ATT_size, LASeR_Size_datatype, 0, GF_XMLNS_SVG },
-	{ "choice", TAG_SVG_ATT_choice, LASeR_Choice_datatype, 0, GF_XMLNS_SVG },
-	{ "delta", TAG_SVG_ATT_delta, LASeR_Size_datatype, 0, GF_XMLNS_SVG },
+	{ "choice", TAG_SVG_ATT_choice, LASeR_Choice_datatype, 0, GF_XMLNS_LASER },
+	{ "delta", TAG_SVG_ATT_delta, LASeR_Size_datatype, 0, GF_XMLNS_LASER },
 	{ "offset", TAG_SVG_ATT_offset, SVG_Number_datatype, 0, GF_XMLNS_SVG },
 	{ "syncBehaviorDefault", TAG_SVG_ATT_syncBehaviorDefault, SMIL_SyncBehavior_datatype, 0, GF_XMLNS_SVG },
 	{ "syncToleranceDefault", TAG_SVG_ATT_syncToleranceDefault, SMIL_SyncTolerance_datatype, 0, GF_XMLNS_SVG },
@@ -286,6 +313,7 @@ static const struct xml_att_def {
 	{ "y", TAG_SVG_ATT_cursorManager_y, SVG_Length_datatype, GF_SVG_ATTOPT_CURSOR, GF_XMLNS_LASER },
 
 	/*XBL*/
+#if 0
 	{ "id", TAG_XBL_ATT_id, DOM_String_datatype, 0, GF_XMLNS_XBL },
 	{ "extends", TAG_XBL_ATT_extends, DOM_String_datatype, 0, GF_XMLNS_XBL },
 	{ "display", TAG_XBL_ATT_display, DOM_String_datatype, 0, GF_XMLNS_XBL },
@@ -309,7 +337,7 @@ static const struct xml_att_def {
 	{ "command", TAG_XBL_ATT_command, DOM_String_datatype, 0, GF_XMLNS_XBL },
 	{ "preventdefault", TAG_XBL_ATT_preventdefault, DOM_String_datatype, 0, GF_XMLNS_XBL },
 	{ "src", TAG_XBL_ATT_src, DOM_String_datatype, 0, GF_XMLNS_XBL },
-
+#endif
 	/*GPAC SVG Extensions*/
 	{ "use-as-primary", TAG_GSVG_ATT_useAsPrimary, SVG_Boolean_datatype, 0, GF_XMLNS_SVG_GPAC_EXTENSION},
 	{ "depthOffset", TAG_GSVG_ATT_depthOffset, SVG_Number_datatype, 0, GF_XMLNS_SVG_GPAC_EXTENSION},
@@ -381,14 +409,13 @@ static char *gf_xml_get_namespace_qname(GF_DOMNode *elt, u32 ns)
 	return gf_xml_get_namespace_qname((GF_DOMNode*)elt->sgprivate->parents->node, ns);
 }
 
-u32 gf_xml_get_attribute_tag(GF_Node *elt, char *attribute_name, u32 ns)
+u32 gf_xml_get_attribute_tag(GF_Node *elt, char *attribute_name, GF_NamespaceType ns)
 {
-	char *ns_sep;
 	u32 i, count;
 	count = sizeof(xml_attributes) / sizeof(struct xml_att_def);
 
 	if (!ns) {
-		ns_sep = strchr(attribute_name, ':');
+		char *ns_sep = strchr(attribute_name, ':');
 		if (ns_sep) {
 			ns_sep[0] = 0;
 			ns = gf_sg_get_namespace_code(elt->sgprivate->scenegraph, attribute_name);
@@ -597,32 +624,13 @@ static const struct xml_elt_def {
 	{ "simpleLayout", TAG_LSR_simpleLayout, GF_XMLNS_LASER },
 	{ "updates", TAG_LSR_updates, GF_XMLNS_LASER },
 
-	/*XBL*/
-	{ "bindings", TAG_XBL_bindings, GF_XMLNS_XBL },
-	{ "binding", TAG_XBL_binding, GF_XMLNS_XBL },
-	{ "content", TAG_XBL_content, GF_XMLNS_XBL },
-	{ "children", TAG_XBL_children, GF_XMLNS_XBL },
-	{ "implementation", TAG_XBL_implementation, GF_XMLNS_XBL },
-	{ "constructor", TAG_XBL_constructor, GF_XMLNS_XBL },
-	{ "destructor", TAG_XBL_destructor, GF_XMLNS_XBL },
-	{ "field", TAG_XBL_field, GF_XMLNS_XBL },
-	{ "property", TAG_XBL_property, GF_XMLNS_XBL },
-	{ "getter", TAG_XBL_getter, GF_XMLNS_XBL },
-	{ "setter", TAG_XBL_setter, GF_XMLNS_XBL },
-	{ "method", TAG_XBL_method, GF_XMLNS_XBL },
-	{ "parameter", TAG_XBL_parameter, GF_XMLNS_XBL },
-	{ "body", TAG_XBL_body, GF_XMLNS_XBL },
-	{ "handlers", TAG_XBL_handlers, GF_XMLNS_XBL },
-	{ "handler", TAG_XBL_handler, GF_XMLNS_XBL },
-	{ "resources", TAG_XBL_resources, GF_XMLNS_XBL },
-	{ "stylesheet", TAG_XBL_stylesheet, GF_XMLNS_XBL },
-	{ "image", TAG_XBL_image, GF_XMLNS_XBL },
 };
 
+GF_EXPORT
 u32 gf_xml_get_element_tag(const char *element_name, u32 ns)
 {
 	u32 i, count;
-	if (!element_name) return TAG_UndefinedNode;
+//	if (!element_name) return TAG_UndefinedNode;
 	count = sizeof(xml_elements) / sizeof(struct xml_elt_def);
 	for (i=0; i<count; i++) {
 		if (!strcmp(xml_elements[i].name, element_name)) {
@@ -642,7 +650,7 @@ const char *gf_xml_get_element_name(GF_Node *n)
 	for (i=0; i<count; i++) {
 		if (n && n->sgprivate && (n->sgprivate->tag==xml_elements[i].tag)) {
 			char *xmlns;
-			if (!n || (ns == xml_elements[i].xmlns))
+			if (ns == xml_elements[i].xmlns)
 				return xml_elements[i].name;
 
 			xmlns = (char *) gf_sg_get_namespace_qname(n->sgprivate->scenegraph, xml_elements[i].xmlns);
@@ -674,8 +682,9 @@ GF_NamespaceType gf_xml_get_element_namespace(GF_Node *n)
 u32 gf_node_get_attribute_count(GF_Node *node)
 {
 	u32 count = 0;
-	GF_DOMNode *dom = (GF_DOMNode *)node;
-	GF_DOMAttribute *atts = dom->attributes;
+	GF_DOMAttribute *atts;
+	if (!node) return 0;
+	atts =  ((GF_DOMNode *)node)->attributes;
 	while (atts) {
 		count++;
 		atts = atts->next;
@@ -702,9 +711,9 @@ GF_Err gf_node_get_attribute_info(GF_Node *node, GF_FieldInfo *info)
 
 void gf_node_delete_attributes(GF_Node *node)
 {
-	GF_DOMAttribute *tmp;
 	GF_DOMAttribute *att = ((GF_DOMNode*)node)->attributes;
 	while(att) {
+		GF_DOMAttribute *tmp;
 		gf_svg_delete_attribute_value(att->data_type, att->data, node->sgprivate->scenegraph);
 		tmp = att;
 		att = att->next;
@@ -956,6 +965,7 @@ void gf_node_register_iri(GF_SceneGraph *sg, XMLRI *target)
 #endif
 }
 
+GF_EXPORT
 void gf_node_unregister_iri(GF_SceneGraph *sg, XMLRI *target)
 {
 #ifndef GPAC_DISABLE_SVG
@@ -966,7 +976,9 @@ void gf_node_unregister_iri(GF_SceneGraph *sg, XMLRI *target)
 GF_Node *gf_xml_node_clone(GF_SceneGraph *inScene, GF_Node *orig, GF_Node *cloned_parent, char *inst_id, Bool deep)
 {
 	GF_DOMAttribute *att;
-	GF_Node *clone = gf_node_new(inScene, orig->sgprivate->tag);
+	GF_Node *clone;
+	if (!orig) return NULL;
+	clone = gf_node_new(inScene, orig->sgprivate->tag);
 	if (!clone) return NULL;
 
 	if (orig->sgprivate->tag == TAG_DOMText) {
@@ -981,7 +993,7 @@ GF_Node *gf_xml_node_clone(GF_SceneGraph *inScene, GF_Node *orig, GF_Node *clone
 			n_src = (GF_DOMFullNode *)orig;
 			n_dst = (GF_DOMFullNode *)clone;
 			n_dst->ns = n_src->ns;
-			n_dst->name = gf_strdup(n_dst->name);
+			n_dst->name = gf_strdup(n_src->name);
 		}
 
 		att = ((GF_DOMNode *)orig)->attributes;
@@ -1039,14 +1051,13 @@ static u32 check_existing_file(char *base_file, char *ext, char *data, u32 data_
 	f = gf_fopen(szFile, "rb");
 	if (!f) return 0;
 
-	gf_fseek(f, 0, SEEK_END);
-	fsize = gf_ftell(f);
+	fsize = gf_fsize(f);
 	if (fsize==data_size) {
 		u32 offset=0;
 		char cache[1024];
-		gf_fseek(f, 0, SEEK_SET);
+
 		while (fsize) {
-			u32 read = (u32) fread(cache, 1, 1024, f);
+			u32 read = (u32) gf_fread(cache, 1024, f);
 			if ((s32) read < 0) return 0;
 			fsize -= read;
 			if (memcmp(cache, data+offset, sizeof(char)*read)) break;
@@ -1066,10 +1077,9 @@ static u32 check_existing_file(char *base_file, char *ext, char *data, u32 data_
 GF_EXPORT
 GF_Err gf_node_store_embedded_data(XMLRI *iri, const char *cache_dir, const char *base_filename)
 {
-	char szFile[GF_MAX_PATH], buf[20], *sep, *data, *ext;
-	u32 data_size, idx;
+	char szFile[GF_MAX_PATH], buf[20], *sep, *data=NULL, *ext;
+	u32 data_size=0, idx;
 	Bool existing;
-	FILE *f;
 
 	if (!cache_dir || !base_filename || !iri || !iri->string || strncmp(iri->string, "data:", 5)) return GF_OK;
 
@@ -1080,16 +1090,16 @@ GF_Err gf_node_store_embedded_data(XMLRI *iri, const char *cache_dir, const char
 		szFile[data_size] = GF_PATH_SEPARATOR;
 		szFile[data_size+1] = 0;
 	}
-	if (base_filename) {
-		sep = strrchr(base_filename, GF_PATH_SEPARATOR);
+
+	sep = strrchr(base_filename, GF_PATH_SEPARATOR);
 #ifdef WIN32
-		if (!sep) sep = strrchr(base_filename, '/');
+	if (!sep) sep = strrchr(base_filename, '/');
 #endif
-		if (!sep) sep = (char *) base_filename;
-		else sep += 1;
-		strcat(szFile, sep);
-	}
-	sep = strrchr(szFile, '.');
+	if (!sep) sep = (char *) base_filename;
+	else sep += 1;
+	strcat(szFile, sep);
+
+	sep = gf_file_ext_start(szFile);
 	if (sep) sep[0] = 0;
 	strcat(szFile, "_img_");
 
@@ -1117,7 +1127,7 @@ GF_Err gf_node_store_embedded_data(XMLRI *iri, const char *cache_dir, const char
 		sep += 8;
 		data_size = gf_base16_decode(sep, (u32) strlen(sep), data, data_size);
 	}
-	if (!data_size) return GF_OK;
+	if (!data || !data_size) return GF_OK;
 
 	iri->type = XMLRI_STRING;
 
@@ -1137,14 +1147,14 @@ GF_Err gf_node_store_embedded_data(XMLRI *iri, const char *cache_dir, const char
 	strcat(szFile, ext);
 
 	if (!existing) {
-		f = gf_fopen(szFile, "wb");
+		FILE *f = gf_fopen(szFile, "wb");
 		if (!f) {
 			gf_free(data);
 			gf_free(iri->string);
 			iri->string = NULL;
 			return GF_IO_ERR;
 		}
-		gf_fwrite(data, data_size, 1, f);
+		gf_fwrite(data, data_size, f);
 		gf_fclose(f);
 	}
 	gf_free(data);
